"use strict";(self["observabilityDashboards_bundle_jsonpfunction"]=self["observabilityDashboards_bundle_jsonpfunction"]||[]).push([["3"],{24643:function(__unused_webpack_module,__webpack_exports__,__webpack_require__){__webpack_require__.r(__webpack_exports__);__webpack_require__.d(__webpack_exports__,{MarkdownSimple:()=>MarkdownSimple,"default":()=>__WEBPACK_DEFAULT_EXPORT__});var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__=__webpack_require__(31085);var react__WEBPACK_IMPORTED_MODULE_1__=__webpack_require__(46933);var react__WEBPACK_IMPORTED_MODULE_1___default=__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);var react_markdown__WEBPACK_IMPORTED_MODULE_2__=__webpack_require__(16245);var react_markdown__WEBPACK_IMPORTED_MODULE_2___default=__webpack_require__.n(react_markdown__WEBPACK_IMPORTED_MODULE_2__);var MarkdownSimple=function(param){var children=param.children;return(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(react__WEBPACK_IMPORTED_MODULE_1__.Fragment,{children:(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(react_markdown__WEBPACK_IMPORTED_MODULE_2___default(),{children:children})})};const __WEBPACK_DEFAULT_EXPORT__=MarkdownSimple}}]);