import { HTMLAttributes, FC } from "react";
interface CommandButtonRowProps extends HTMLAttributes<HTMLButtonElement> {
    shortCut: string[];
}
export declare const CommandButtonRow: FC<CommandButtonRowProps>;
export {};
