/** Alignment along the horizontal axis. */
export declare const Alignment: {
    CENTER: "center";
    LEFT: "left";
    RIGHT: "right";
};
export declare type Alignment = typeof Alignment[keyof typeof Alignment];
