import * as React from "react";
interface OutputsProps {
    className?: string;
    children?: React.ReactNode;
    /**
     * Whether or not to collapse the children of the component.
     */
    expanded?: boolean;
    /**
     * Whether or not to show the children of the component.
     */
    hidden?: boolean;
}
export declare class Outputs extends React.PureComponent<OutputsProps> {
    static defaultProps: {
        children: null;
        className: string;
        hidden: boolean;
        expanded: boolean;
    };
    render(): JSX.Element | null;
}
export default Outputs;
