"use strict";
/*
 * Copyright 2021 Palantir Technologies, Inc. All rights reserved.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CAMERA = exports.CALENDAR = exports.CALCULATOR = exports.BUILD = exports.BRING_DATA = exports.BRIEFCASE = exports.BOX = exports.BOOKMARK = exports.BOOK = exports.BOLD = exports.BLOCKED_PERSON = exports.BLANK = exports.BARCODE = exports.BANK_ACCOUNT = exports.BAN_CIRCLE = exports.BADGE = exports.AUTOMATIC_UPDATES = exports.ASTERISK = exports.ARROWS_VERTICAL = exports.ARROWS_HORIZONTAL = exports.ARROW_UP = exports.ARROW_TOP_RIGHT = exports.ARROW_TOP_LEFT = exports.ARROW_RIGHT = exports.ARROW_LEFT = exports.ARROW_DOWN = exports.ARROW_BOTTOM_RIGHT = exports.ARROW_BOTTOM_LEFT = exports.ARCHIVE = exports.APPLICATIONS = exports.APPLICATION = exports.ANNOTATION = exports.ALIGNMENT_VERTICAL_CENTER = exports.ALIGNMENT_TOP = exports.ALIGNMENT_RIGHT = exports.ALIGNMENT_LEFT = exports.ALIGNMENT_HORIZONTAL_CENTER = exports.ALIGNMENT_BOTTOM = exports.ALIGN_RIGHT = exports.ALIGN_LEFT = exports.ALIGN_JUSTIFY = exports.ALIGN_CENTER = exports.AIRPLANE = exports.ADD_TO_FOLDER = exports.ADD_TO_ARTIFACT = exports.ADD_ROW_TOP = exports.ADD_ROW_BOTTOM = exports.ADD_COLUMN_RIGHT = exports.ADD_COLUMN_LEFT = exports.ADD = void 0;
exports.DELETE = exports.DATABASE = exports.DATA_LINEAGE = exports.DASHBOARD = exports.CUT = exports.CURVED_RANGE_CHART = exports.CUBE_REMOVE = exports.CUBE_ADD = exports.CUBE = exports.CROWN = exports.CROSS = exports.CREDIT_CARD = exports.CONTROL = exports.CONTRAST = exports.CONSOLE = exports.CONFIRM = exports.COMPRESSED = exports.COMPASS = exports.COMPARISON = exports.COMMENT = exports.COLUMN_LAYOUT = exports.COLLAPSE_ALL = exports.COG = exports.CODE_BLOCK = exports.CODE = exports.CLOUD_UPLOAD = exports.CLOUD_DOWNLOAD = exports.CLOUD = exports.CLIPBOARD = exports.CLEAN = exports.CITATION = exports.CIRCLE_ARROW_UP = exports.CIRCLE_ARROW_RIGHT = exports.CIRCLE_ARROW_LEFT = exports.CIRCLE_ARROW_DOWN = exports.CIRCLE = exports.CHEVRON_UP = exports.CHEVRON_RIGHT = exports.CHEVRON_LEFT = exports.CHEVRON_FORWARD = exports.CHEVRON_DOWN = exports.CHEVRON_BACKWARD = exports.CHAT = exports.CHART = exports.CHANGES = exports.CELL_TOWER = exports.CARET_UP = exports.CARET_RIGHT = exports.CARET_LEFT = exports.CARET_DOWN = void 0;
exports.FILTER_OPEN = exports.FILTER_LIST = exports.FILTER_KEEP = exports.FILTER = exports.FILM = exports.FEED_SUBSCRIBED = exports.FEED = exports.FAST_FORWARD = exports.FAST_BACKWARD = exports.EYE_OPEN = exports.EYE_ON = exports.EYE_OFF = exports.EXPORT = exports.EXPAND_ALL = exports.EXCLUDE_ROW = exports.EXCHANGE = exports.EURO = exports.ERROR = exports.ERASER = exports.EQUALS = exports.ENVELOPE = exports.ENDORSED = exports.EJECT = exports.EDIT = exports.DUPLICATE = exports.DRIVE_TIME = exports.DRAW = exports.DRAG_HANDLE_VERTICAL = exports.DRAG_HANDLE_HORIZONTAL = exports.DOWNLOAD = exports.DOUGHNUT_CHART = exports.DOUBLE_CHEVRON_UP = exports.DOUBLE_CHEVRON_RIGHT = exports.DOUBLE_CHEVRON_LEFT = exports.DOUBLE_CHEVRON_DOWN = exports.DOUBLE_CARET_VERTICAL = exports.DOUBLE_CARET_HORIZONTAL = exports.DOT = exports.DOLLAR = exports.DOCUMENT_SHARE = exports.DOCUMENT_OPEN = exports.DOCUMENT = exports.DISABLE = exports.DIRECTION_RIGHT = exports.DIRECTION_LEFT = exports.DIAGRAM_TREE = exports.DIAGNOSIS = exports.DESKTOP = exports.DERIVE_COLUMN = exports.DELTA = void 0;
exports.HAND_RIGHT = exports.HAND_LEFT = exports.HAND_DOWN = exports.HAND = exports.GROUPED_BAR_CHART = exports.GROUP_OBJECTS = exports.GRID_VIEW = exports.GRID = exports.GREATER_THAN_OR_EQUAL_TO = exports.GREATER_THAN = exports.GRAPH_REMOVE = exports.GRAPH = exports.GLOBE_NETWORK = exports.GLOBE = exports.GLASS = exports.GIT_REPO = exports.GIT_PUSH = exports.GIT_PULL = exports.GIT_NEW_BRANCH = exports.GIT_MERGE = exports.GIT_COMMIT = exports.GIT_BRANCH = exports.GEOSEARCH = exports.GEOLOCATION = exports.GANTT_CHART = exports.FUNCTION = exports.FULLSCREEN = exports.FULL_STACKED_CHART = exports.FULL_CIRCLE = exports.FORM = exports.FORK = exports.FONT = exports.FOLLOWING = exports.FOLLOWER = exports.FOLDER_SHARED_OPEN = exports.FOLDER_SHARED = exports.FOLDER_OPEN = exports.FOLDER_NEW = exports.FOLDER_CLOSE = exports.FLOWS = exports.FLOW_REVIEW_BRANCH = exports.FLOW_REVIEW = exports.FLOW_LINEAR = exports.FLOW_END = exports.FLOW_BRANCH = exports.FLOPPY_DISK = exports.FLASH = exports.FLAME = exports.FLAG = exports.FILTER_REMOVE = void 0;
exports.LAB_TEST = exports.KNOWN_VEHICLE = exports.KEY_TAB = exports.KEY_SHIFT = exports.KEY_OPTION = exports.KEY_ESCAPE = exports.KEY_ENTER = exports.KEY_DELETE = exports.KEY_CONTROL = exports.KEY_COMMAND = exports.KEY_BACKSPACE = exports.KEY = exports.JOIN_TABLE = exports.ITALIC = exports.ISSUE_NEW = exports.ISSUE_CLOSED = exports.ISSUE = exports.IP_ADDRESS = exports.INTERSECTION = exports.INSERT = exports.INNER_JOIN = exports.INHERITANCE = exports.INFO_SIGN = exports.INBOX_UPDATE = exports.INBOX_SEARCH = exports.INBOX_GEO = exports.INBOX_FILTERED = exports.INBOX = exports.IMPORT = exports.IMAGE_ROTATE_RIGHT = exports.IMAGE_ROTATE_LEFT = exports.ID_NUMBER = exports.HORIZONTAL_DISTRIBUTION = exports.HORIZONTAL_BAR_CHART_DESC = exports.HORIZONTAL_BAR_CHART_ASC = exports.HORIZONTAL_BAR_CHART = exports.HOME = exports.HISTORY = exports.HIGHLIGHT = exports.HELPER_MANAGEMENT = exports.HELP = exports.HEATMAP = exports.HEAT_GRID = exports.HEART_BROKEN = exports.HEART = exports.HEADSET = exports.HEADER_TWO = exports.HEADER_ONE = exports.HEADER = exports.HAND_UP = void 0;
exports.MUSIC = exports.MULTI_SELECT = exports.MUGSHOT = exports.MOVE = exports.MOUNTAIN = exports.MORE = exports.MOON = exports.MOBILE_VIDEO = exports.MOBILE_PHONE = exports.MINUS = exports.MINIMIZE = exports.MERGE_LINKS = exports.MERGE_COLUMNS = exports.MENU_OPEN = exports.MENU_CLOSED = exports.MENU = exports.MEDIA = exports.MAXIMIZE = exports.MAP_MARKER = exports.MAP_CREATE = exports.MAP = exports.MANUALLY_ENTERED_DATA = exports.MANUAL = exports.LOG_OUT = exports.LOG_IN = exports.LOCK = exports.LOCATE = exports.LIST_DETAIL_VIEW = exports.LIST_COLUMNS = exports.LIST = exports.LINK = exports.LIGHTBULB = exports.LIFESAVER = exports.LESS_THAN_OR_EQUAL_TO = exports.LESS_THAN = exports.LEFT_JOIN = exports.LEARNING = exports.LAYOUT_SORTED_CLUSTERS = exports.LAYOUT_SKEW_GRID = exports.LAYOUT_LINEAR = exports.LAYOUT_HIERARCHY = exports.LAYOUT_GROUP_BY = exports.LAYOUT_GRID = exports.LAYOUT_CIRCLE = exports.LAYOUT_BALLOON = exports.LAYOUT_AUTO = exports.LAYOUT = exports.LAYERS = exports.LAYER = exports.LABEL = void 0;
exports.RANDOM = exports.PULSE = exports.PUBLISH_FUNCTION = exports.PROPERTY = exports.PROPERTIES = exports.PROJECTS = exports.PRINT = exports.PRESENTATION = exports.PRESCRIPTION = exports.PREDICTIVE_ANALYSIS = exports.POWER = exports.POLYGON_FILTER = exports.PLUS = exports.PLAY = exports.PIVOT_TABLE = exports.PIVOT = exports.PIN = exports.PIE_CHART = exports.PHONE = exports.PERSON = exports.PERCENTAGE = exports.PEOPLE = exports.PAUSE = exports.PATH_SEARCH = exports.PATH = exports.PARAGRAPH = exports.PAPERCLIP = exports.PANEL_TABLE = exports.PANEL_STATS = exports.PAGE_LAYOUT = exports.OUTDATED = exports.ONE_COLUMN = exports.OIL_FIELD = exports.OFFLINE = exports.OFFICE = exports.NUMERICAL = exports.NUMBERED_LIST = exports.NOTIFICATIONS_UPDATED = exports.NOTIFICATIONS = exports.NOT_EQUAL_TO = exports.NINJA = exports.NEW_TEXT_BOX = exports.NEW_PRESCRIPTION = exports.NEW_PERSON = exports.NEW_OBJECT = exports.NEW_LINK = exports.NEW_LAYERS = exports.NEW_LAYER = exports.NEW_GRID_ITEM = exports.NEW_DRAWING = void 0;
exports.SOCIAL_MEDIA = exports.SNOWFLAKE = exports.SMALL_TICK = exports.SMALL_PLUS = exports.SMALL_MINUS = exports.SMALL_CROSS = exports.SLASH = exports.SIM_CARD = exports.SIGNAL_SEARCH = exports.SHOPPING_CART = exports.SHOP = exports.SHIELD = exports.SHARE = exports.SETTINGS = exports.SERIES_SEARCH = exports.SERIES_FILTERED = exports.SERIES_DERIVED = exports.SERIES_CONFIGURATION = exports.SERIES_ADD = exports.SEND_TO_MAP = exports.SEND_TO_GRAPH = exports.SEND_TO = exports.SELECTION = exports.SELECT = exports.SEGMENTED_CONTROL = exports.SEARCH_TEXT = exports.SEARCH_TEMPLATE = exports.SEARCH_AROUND = exports.SEARCH = exports.SCATTER_PLOT = exports.SAVED = exports.SATELLITE = exports.ROTATE_PAGE = exports.ROTATE_DOCUMENT = exports.RING = exports.RIGHT_JOIN = exports.RIG = exports.RESOLVE = exports.RESET = exports.REPEAT = exports.REMOVE_ROW_TOP = exports.REMOVE_ROW_BOTTOM = exports.REMOVE_COLUMN_RIGHT = exports.REMOVE_COLUMN_LEFT = exports.REMOVE_COLUMN = exports.REMOVE = exports.REGRESSION_CHART = exports.REFRESH = exports.REDO = exports.RECORD = void 0;
exports.TRANSLATE = exports.TRAIN = exports.TRACTOR = exports.TORCH = exports.TINT = exports.TIMELINE_LINE_CHART = exports.TIMELINE_EVENTS = exports.TIMELINE_BAR_CHART = exports.TIMELINE_AREA_CHART = exports.TIME = exports.TICK_CIRCLE = exports.TICK = exports.THUMBS_UP = exports.THUMBS_DOWN = exports.TH_LIST = exports.TH_FILTERED = exports.TH_DISCONNECT = exports.TH_DERIVED = exports.TH = exports.TEXT_HIGHLIGHT = exports.TAXI = exports.TAKE_ACTION = exports.TAG = exports.SYMBOL_TRIANGLE_UP = exports.SYMBOL_TRIANGLE_DOWN = exports.SYMBOL_SQUARE = exports.SYMBOL_DIAMOND = exports.SYMBOL_CROSS = exports.SYMBOL_CIRCLE = exports.SWAP_VERTICAL = exports.SWAP_HORIZONTAL = exports.STYLE = exports.STRIKETHROUGH = exports.STOPWATCH = exports.STOP = exports.STEP_FORWARD = exports.STEP_CHART = exports.STEP_BACKWARD = exports.STAR_EMPTY = exports.STAR = exports.STACKED_CHART = exports.SQUARE = exports.SPLIT_COLUMNS = exports.SORT_NUMERICAL_DESC = exports.SORT_NUMERICAL = exports.SORT_DESC = exports.SORT_ASC = exports.SORT_ALPHABETICAL_DESC = exports.SORT_ALPHABETICAL = exports.SORT = void 0;
exports.ZOOM_TO_FIT = exports.ZOOM_OUT = exports.ZOOM_IN = exports.WRENCH = exports.WIDGET_HEADER = exports.WIDGET_FOOTER = exports.WIDGET_BUTTON = exports.WIDGET = exports.WATERFALL_CHART = exports.WARNING_SIGN = exports.WALK = exports.VOLUME_UP = exports.VOLUME_OFF = exports.VOLUME_DOWN = exports.VIDEO = exports.VERTICAL_DISTRIBUTION = exports.VERTICAL_BAR_CHART_DESC = exports.VERTICAL_BAR_CHART_ASC = exports.VARIABLE = exports.USER = exports.UPLOAD = exports.UPDATED = exports.UNRESOLVE = exports.UNPIN = exports.UNLOCK = exports.UNKNOWN_VEHICLE = exports.UNGROUP_OBJECTS = exports.UNDO = exports.UNDERLINE = exports.UNARCHIVE = exports.TWO_COLUMNS = exports.TRUCK = exports.TRENDING_UP = exports.TRENDING_DOWN = exports.TREE = exports.TRASH = void 0;
exports.ADD = "\ue63e";
exports.ADD_COLUMN_LEFT = "\ue6f9";
exports.ADD_COLUMN_RIGHT = "\ue6fa";
exports.ADD_ROW_BOTTOM = "\ue6f8";
exports.ADD_ROW_TOP = "\ue6f7";
exports.ADD_TO_ARTIFACT = "\ue67c";
exports.ADD_TO_FOLDER = "\ue6d2";
exports.AIRPLANE = "\ue74b";
exports.ALIGN_CENTER = "\ue603";
exports.ALIGN_JUSTIFY = "\ue605";
exports.ALIGN_LEFT = "\ue602";
exports.ALIGN_RIGHT = "\ue604";
exports.ALIGNMENT_BOTTOM = "\ue727";
exports.ALIGNMENT_HORIZONTAL_CENTER = "\ue726";
exports.ALIGNMENT_LEFT = "\ue722";
exports.ALIGNMENT_RIGHT = "\ue724";
exports.ALIGNMENT_TOP = "\ue725";
exports.ALIGNMENT_VERTICAL_CENTER = "\ue723";
exports.ANNOTATION = "\ue6f0";
exports.APPLICATION = "\ue735";
exports.APPLICATIONS = "\ue621";
exports.ARCHIVE = "\ue907";
exports.ARROW_BOTTOM_LEFT = "\u2199";
exports.ARROW_BOTTOM_RIGHT = "\u2198";
exports.ARROW_DOWN = "\u2193";
exports.ARROW_LEFT = "\u2190";
exports.ARROW_RIGHT = "\u2192";
exports.ARROW_TOP_LEFT = "\u2196";
exports.ARROW_TOP_RIGHT = "\u2197";
exports.ARROW_UP = "\u2191";
exports.ARROWS_HORIZONTAL = "\u2194";
exports.ARROWS_VERTICAL = "\u2195";
exports.ASTERISK = "\u002a";
exports.AUTOMATIC_UPDATES = "\ue65f";
exports.BADGE = "\ue6e3";
exports.BAN_CIRCLE = "\ue69d";
exports.BANK_ACCOUNT = "\ue76f";
exports.BARCODE = "\ue676";
exports.BLANK = "\ue900";
exports.BLOCKED_PERSON = "\ue768";
exports.BOLD = "\ue606";
exports.BOOK = "\ue6b8";
exports.BOOKMARK = "\ue61a";
exports.BOX = "\ue6bf";
exports.BRIEFCASE = "\ue674";
exports.BRING_DATA = "\ue90a";
exports.BUILD = "\ue72d";
exports.CALCULATOR = "\ue70b";
exports.CALENDAR = "\ue62b";
exports.CAMERA = "\ue69e";
exports.CARET_DOWN = "\u2304";
exports.CARET_LEFT = "\u2329";
exports.CARET_RIGHT = "\u232A";
exports.CARET_UP = "\u2303";
exports.CELL_TOWER = "\ue770";
exports.CHANGES = "\ue623";
exports.CHART = "\ue67e";
exports.CHAT = "\ue689";
exports.CHEVRON_BACKWARD = "\ue6df";
exports.CHEVRON_DOWN = "\ue697";
exports.CHEVRON_FORWARD = "\ue6e0";
exports.CHEVRON_LEFT = "\ue694";
exports.CHEVRON_RIGHT = "\ue695";
exports.CHEVRON_UP = "\ue696";
exports.CIRCLE = "\ue66a";
exports.CIRCLE_ARROW_DOWN = "\ue68e";
exports.CIRCLE_ARROW_LEFT = "\ue68c";
exports.CIRCLE_ARROW_RIGHT = "\ue68b";
exports.CIRCLE_ARROW_UP = "\ue68d";
exports.CITATION = "\ue61b";
exports.CLEAN = "\ue7c5";
exports.CLIPBOARD = "\ue61d";
exports.CLOUD = "\u2601";
exports.CLOUD_DOWNLOAD = "\ue690";
exports.CLOUD_UPLOAD = "\ue691";
exports.CODE = "\ue661";
exports.CODE_BLOCK = "\ue6c5";
exports.COG = "\ue645";
exports.COLLAPSE_ALL = "\ue763";
exports.COLUMN_LAYOUT = "\ue6da";
exports.COMMENT = "\ue68a";
exports.COMPARISON = "\ue637";
exports.COMPASS = "\ue79c";
exports.COMPRESSED = "\ue6c0";
exports.CONFIRM = "\ue639";
exports.CONSOLE = "\ue79b";
exports.CONTRAST = "\ue6cb";
exports.CONTROL = "\ue67f";
exports.CREDIT_CARD = "\ue649";
exports.CROSS = "\u2717";
exports.CROWN = "\ue7b4";
exports.CUBE = "\ue7c8";
exports.CUBE_ADD = "\ue7c9";
exports.CUBE_REMOVE = "\ue7d0";
exports.CURVED_RANGE_CHART = "\ue71b";
exports.CUT = "\ue6ef";
exports.DASHBOARD = "\ue751";
exports.DATA_LINEAGE = "\ue908";
exports.DATABASE = "\ue683";
exports.DELETE = "\ue644";
exports.DELTA = "\u0394";
exports.DERIVE_COLUMN = "\ue739";
exports.DESKTOP = "\ue6af";
exports.DIAGNOSIS = "\ue90d";
exports.DIAGRAM_TREE = "\ue7b3";
exports.DIRECTION_LEFT = "\ue681";
exports.DIRECTION_RIGHT = "\ue682";
exports.DISABLE = "\ue600";
exports.DOCUMENT = "\ue630";
exports.DOCUMENT_OPEN = "\ue71e";
exports.DOCUMENT_SHARE = "\ue71f";
exports.DOLLAR = "\u0024";
exports.DOT = "\u2022";
exports.DOUBLE_CARET_HORIZONTAL = "\ue6c7";
exports.DOUBLE_CARET_VERTICAL = "\ue6c6";
exports.DOUBLE_CHEVRON_DOWN = "\ue703";
exports.DOUBLE_CHEVRON_LEFT = "\ue6ff";
exports.DOUBLE_CHEVRON_RIGHT = "\ue701";
exports.DOUBLE_CHEVRON_UP = "\ue702";
exports.DOUGHNUT_CHART = "\ue6ce";
exports.DOWNLOAD = "\ue62f";
exports.DRAG_HANDLE_HORIZONTAL = "\ue716";
exports.DRAG_HANDLE_VERTICAL = "\ue715";
exports.DRAW = "\ue66b";
exports.DRIVE_TIME = "\ue615";
exports.DUPLICATE = "\ue69c";
exports.EDIT = "\u270E";
exports.EJECT = "\u23cf";
exports.ENDORSED = "\ue75f";
exports.ENVELOPE = "\u2709";
exports.EQUALS = "\ue7d9";
exports.ERASER = "\ue773";
exports.ERROR = "\ue648";
exports.EURO = "\u20ac";
exports.EXCHANGE = "\ue636";
exports.EXCLUDE_ROW = "\ue6ea";
exports.EXPAND_ALL = "\ue764";
exports.EXPORT = "\ue633";
exports.EYE_OFF = "\ue6cc";
exports.EYE_ON = "\ue75a";
exports.EYE_OPEN = "\ue66f";
exports.FAST_BACKWARD = "\ue6a8";
exports.FAST_FORWARD = "\ue6ac";
exports.FEED = "\ue656";
exports.FEED_SUBSCRIBED = "\ue78f";
exports.FILM = "\ue6a1";
exports.FILTER = "\ue638";
exports.FILTER_KEEP = "\ue78c";
exports.FILTER_LIST = "\ue6ee";
exports.FILTER_OPEN = "\ue7d7";
exports.FILTER_REMOVE = "\ue78d";
exports.FLAG = "\u2691";
exports.FLAME = "\ue7a9";
exports.FLASH = "\ue6b3";
exports.FLOPPY_DISK = "\ue6b7";
exports.FLOW_BRANCH = "\ue7c1";
exports.FLOW_END = "\ue7c4";
exports.FLOW_LINEAR = "\ue7c0";
exports.FLOW_REVIEW = "\ue7c2";
exports.FLOW_REVIEW_BRANCH = "\ue7c3";
exports.FLOWS = "\ue659";
exports.FOLDER_CLOSE = "\ue652";
exports.FOLDER_NEW = "\ue7b0";
exports.FOLDER_OPEN = "\ue651";
exports.FOLDER_SHARED = "\ue653";
exports.FOLDER_SHARED_OPEN = "\ue670";
exports.FOLLOWER = "\ue760";
exports.FOLLOWING = "\ue761";
exports.FONT = "\ue6b4";
exports.FORK = "\ue63a";
exports.FORM = "\ue795";
exports.FULL_CIRCLE = "\ue685";
exports.FULL_STACKED_CHART = "\ue75e";
exports.FULLSCREEN = "\ue699";
exports.FUNCTION = "\ue6e5";
exports.GANTT_CHART = "\ue6f4";
exports.GEOLOCATION = "\ue640";
exports.GEOSEARCH = "\ue613";
exports.GIT_BRANCH = "\ue72a";
exports.GIT_COMMIT = "\ue72b";
exports.GIT_MERGE = "\ue729";
exports.GIT_NEW_BRANCH = "\ue749";
exports.GIT_PULL = "\ue728";
exports.GIT_PUSH = "\ue72c";
exports.GIT_REPO = "\ue748";
exports.GLASS = "\ue6b1";
exports.GLOBE = "\ue666";
exports.GLOBE_NETWORK = "\ue7b5";
exports.GRAPH = "\ue673";
exports.GRAPH_REMOVE = "\ue609";
exports.GREATER_THAN = "\ue7e1";
exports.GREATER_THAN_OR_EQUAL_TO = "\ue7e2";
exports.GRID = "\ue6d0";
exports.GRID_VIEW = "\ue6e4";
exports.GROUP_OBJECTS = "\ue60a";
exports.GROUPED_BAR_CHART = "\ue75d";
exports.HAND = "\ue6de";
exports.HAND_DOWN = "\ue6bb";
exports.HAND_LEFT = "\ue6bc";
exports.HAND_RIGHT = "\ue6b9";
exports.HAND_UP = "\ue6ba";
exports.HEADER = "\ue6b5";
exports.HEADER_ONE = "\ue793";
exports.HEADER_TWO = "\ue794";
exports.HEADSET = "\ue6dc";
exports.HEART = "\u2665";
exports.HEART_BROKEN = "\ue7a2";
exports.HEAT_GRID = "\ue6f3";
exports.HEATMAP = "\ue614";
exports.HELP = "\u003F";
exports.HELPER_MANAGEMENT = "\ue66d";
exports.HIGHLIGHT = "\ue6ed";
exports.HISTORY = "\ue64a";
exports.HOME = "\u2302";
exports.HORIZONTAL_BAR_CHART = "\ue70c";
exports.HORIZONTAL_BAR_CHART_ASC = "\ue75c";
exports.HORIZONTAL_BAR_CHART_DESC = "\ue71d";
exports.HORIZONTAL_DISTRIBUTION = "\ue720";
exports.ID_NUMBER = "\ue771";
exports.IMAGE_ROTATE_LEFT = "\ue73a";
exports.IMAGE_ROTATE_RIGHT = "\ue73b";
exports.IMPORT = "\ue632";
exports.INBOX = "\ue629";
exports.INBOX_FILTERED = "\ue7d1";
exports.INBOX_GEO = "\ue7d2";
exports.INBOX_SEARCH = "\ue7d3";
exports.INBOX_UPDATE = "\ue7d4";
exports.INFO_SIGN = "\u2139";
exports.INHERITANCE = "\ue7d5";
exports.INNER_JOIN = "\ue7a3";
exports.INSERT = "\ue66c";
exports.INTERSECTION = "\ue765";
exports.IP_ADDRESS = "\ue772";
exports.ISSUE = "\ue774";
exports.ISSUE_CLOSED = "\ue776";
exports.ISSUE_NEW = "\ue775";
exports.ITALIC = "\ue607";
exports.JOIN_TABLE = "\ue738";
exports.KEY = "\ue78e";
exports.KEY_BACKSPACE = "\ue707";
exports.KEY_COMMAND = "\ue705";
exports.KEY_CONTROL = "\ue704";
exports.KEY_DELETE = "\ue708";
exports.KEY_ENTER = "\ue70a";
exports.KEY_ESCAPE = "\ue709";
exports.KEY_OPTION = "\ue742";
exports.KEY_SHIFT = "\ue706";
exports.KEY_TAB = "\ue757";
exports.KNOWN_VEHICLE = "\ue73c";
exports.LAB_TEST = "\ue90e";
exports.LABEL = "\ue665";
exports.LAYER = "\ue6cf";
exports.LAYERS = "\ue618";
exports.LAYOUT = "\ue60c";
exports.LAYOUT_AUTO = "\ue60d";
exports.LAYOUT_BALLOON = "\ue6d3";
exports.LAYOUT_CIRCLE = "\ue60e";
exports.LAYOUT_GRID = "\ue610";
exports.LAYOUT_GROUP_BY = "\ue611";
exports.LAYOUT_HIERARCHY = "\ue60f";
exports.LAYOUT_LINEAR = "\ue6c3";
exports.LAYOUT_SKEW_GRID = "\ue612";
exports.LAYOUT_SORTED_CLUSTERS = "\ue6d4";
exports.LEARNING = "\ue904";
exports.LEFT_JOIN = "\ue7a4";
exports.LESS_THAN = "\ue7e3";
exports.LESS_THAN_OR_EQUAL_TO = "\ue7e4";
exports.LIFESAVER = "\ue7c7";
exports.LIGHTBULB = "\ue6b0";
exports.LINK = "\ue62d";
exports.LIST = "\u2630";
exports.LIST_COLUMNS = "\ue7b9";
exports.LIST_DETAIL_VIEW = "\ue743";
exports.LOCATE = "\ue619";
exports.LOCK = "\ue625";
exports.LOG_IN = "\ue69a";
exports.LOG_OUT = "\ue64c";
exports.MANUAL = "\ue6f6";
exports.MANUALLY_ENTERED_DATA = "\ue74a";
exports.MAP = "\ue662";
exports.MAP_CREATE = "\ue741";
exports.MAP_MARKER = "\ue67d";
exports.MAXIMIZE = "\ue635";
exports.MEDIA = "\ue62c";
exports.MENU = "\ue762";
exports.MENU_CLOSED = "\ue655";
exports.MENU_OPEN = "\ue654";
exports.MERGE_COLUMNS = "\ue74f";
exports.MERGE_LINKS = "\ue60b";
exports.MINIMIZE = "\ue634";
exports.MINUS = "\u2212";
exports.MOBILE_PHONE = "\ue717";
exports.MOBILE_VIDEO = "\ue69f";
exports.MOON = "\ue754";
exports.MORE = "\ue62a";
exports.MOUNTAIN = "\ue7b1";
exports.MOVE = "\ue693";
exports.MUGSHOT = "\ue6db";
exports.MULTI_SELECT = "\ue680";
exports.MUSIC = "\ue6a6";
exports.NEW_DRAWING = "\ue905";
exports.NEW_GRID_ITEM = "\ue747";
exports.NEW_LAYER = "\ue902";
exports.NEW_LAYERS = "\ue903";
exports.NEW_LINK = "\ue65c";
exports.NEW_OBJECT = "\ue65d";
exports.NEW_PERSON = "\ue6e9";
exports.NEW_PRESCRIPTION = "\ue78b";
exports.NEW_TEXT_BOX = "\ue65b";
exports.NINJA = "\ue675";
exports.NOT_EQUAL_TO = "\ue7e0";
exports.NOTIFICATIONS = "\ue624";
exports.NOTIFICATIONS_UPDATED = "\ue7b8";
exports.NUMBERED_LIST = "\ue746";
exports.NUMERICAL = "\ue756";
exports.OFFICE = "\ue69b";
exports.OFFLINE = "\ue67a";
exports.OIL_FIELD = "\ue73f";
exports.ONE_COLUMN = "\ue658";
exports.OUTDATED = "\ue7a8";
exports.PAGE_LAYOUT = "\ue660";
exports.PANEL_STATS = "\ue777";
exports.PANEL_TABLE = "\ue778";
exports.PAPERCLIP = "\ue664";
exports.PARAGRAPH = "\ue76c";
exports.PATH = "\ue753";
exports.PATH_SEARCH = "\ue65e";
exports.PAUSE = "\ue6a9";
exports.PEOPLE = "\ue63d";
exports.PERCENTAGE = "\ue76a";
exports.PERSON = "\ue63c";
exports.PHONE = "\u260e";
exports.PIE_CHART = "\ue684";
exports.PIN = "\ue646";
exports.PIVOT = "\ue6f1";
exports.PIVOT_TABLE = "\ue6eb";
exports.PLAY = "\ue6ab";
exports.PLUS = "\u002b";
exports.POLYGON_FILTER = "\ue6d1";
exports.POWER = "\ue6d9";
exports.PREDICTIVE_ANALYSIS = "\ue617";
exports.PRESCRIPTION = "\ue78a";
exports.PRESENTATION = "\ue687";
exports.PRINT = "\u2399";
exports.PROJECTS = "\ue622";
exports.PROPERTIES = "\ue631";
exports.PROPERTY = "\ue65a";
exports.PUBLISH_FUNCTION = "\ue752";
exports.PULSE = "\ue6e8";
exports.RANDOM = "\ue698";
exports.RECORD = "\ue6ae";
exports.REDO = "\ue6c4";
exports.REFRESH = "\ue643";
exports.REGRESSION_CHART = "\ue758";
exports.REMOVE = "\ue63f";
exports.REMOVE_COLUMN = "\ue755";
exports.REMOVE_COLUMN_LEFT = "\ue6fd";
exports.REMOVE_COLUMN_RIGHT = "\ue6fe";
exports.REMOVE_ROW_BOTTOM = "\ue6fc";
exports.REMOVE_ROW_TOP = "\ue6fb";
exports.REPEAT = "\ue692";
exports.RESET = "\ue7d6";
exports.RESOLVE = "\ue672";
exports.RIG = "\ue740";
exports.RIGHT_JOIN = "\ue7a5";
exports.RING = "\ue6f2";
exports.ROTATE_DOCUMENT = "\ue6e1";
exports.ROTATE_PAGE = "\ue6e2";
exports.SATELLITE = "\ue76b";
exports.SAVED = "\ue6b6";
exports.SCATTER_PLOT = "\ue73e";
exports.SEARCH = "\ue64b";
exports.SEARCH_AROUND = "\ue608";
exports.SEARCH_TEMPLATE = "\ue628";
exports.SEARCH_TEXT = "\ue663";
exports.SEGMENTED_CONTROL = "\ue6ec";
exports.SELECT = "\ue616";
exports.SELECTION = "\u29bf";
exports.SEND_TO = "\ue66e";
exports.SEND_TO_GRAPH = "\ue736";
exports.SEND_TO_MAP = "\ue737";
exports.SERIES_ADD = "\ue796";
exports.SERIES_CONFIGURATION = "\ue79a";
exports.SERIES_DERIVED = "\ue799";
exports.SERIES_FILTERED = "\ue798";
exports.SERIES_SEARCH = "\ue797";
exports.SETTINGS = "\ue6a2";
exports.SHARE = "\ue62e";
exports.SHIELD = "\ue7b2";
exports.SHOP = "\ue6c2";
exports.SHOPPING_CART = "\ue6c1";
exports.SIGNAL_SEARCH = "\ue909";
exports.SIM_CARD = "\ue718";
exports.SLASH = "\ue769";
exports.SMALL_CROSS = "\ue6d7";
exports.SMALL_MINUS = "\ue70e";
exports.SMALL_PLUS = "\ue70d";
exports.SMALL_TICK = "\ue6d8";
exports.SNOWFLAKE = "\ue7b6";
exports.SOCIAL_MEDIA = "\ue671";
exports.SORT = "\ue64f";
exports.SORT_ALPHABETICAL = "\ue64d";
exports.SORT_ALPHABETICAL_DESC = "\ue6c8";
exports.SORT_ASC = "\ue6d5";
exports.SORT_DESC = "\ue6d6";
exports.SORT_NUMERICAL = "\ue64e";
exports.SORT_NUMERICAL_DESC = "\ue6c9";
exports.SPLIT_COLUMNS = "\ue750";
exports.SQUARE = "\ue686";
exports.STACKED_CHART = "\ue6e7";
exports.STAR = "\u2605";
exports.STAR_EMPTY = "\u2606";
exports.STEP_BACKWARD = "\ue6a7";
exports.STEP_CHART = "\ue70f";
exports.STEP_FORWARD = "\ue6ad";
exports.STOP = "\ue6aa";
exports.STOPWATCH = "\ue901";
exports.STRIKETHROUGH = "\ue7a6";
exports.STYLE = "\ue601";
exports.SWAP_HORIZONTAL = "\ue745";
exports.SWAP_VERTICAL = "\ue744";
exports.SYMBOL_CIRCLE = "\ue72e";
exports.SYMBOL_CROSS = "\ue731";
exports.SYMBOL_DIAMOND = "\ue730";
exports.SYMBOL_SQUARE = "\ue72f";
exports.SYMBOL_TRIANGLE_DOWN = "\ue733";
exports.SYMBOL_TRIANGLE_UP = "\ue732";
exports.TAG = "\ue61c";
exports.TAKE_ACTION = "\ue6ca";
exports.TAXI = "\ue79e";
exports.TEXT_HIGHLIGHT = "\ue6dd";
exports.TH = "\ue667";
exports.TH_DERIVED = "\ue669";
exports.TH_DISCONNECT = "\ue7d8";
exports.TH_FILTERED = "\ue7c6";
exports.TH_LIST = "\ue668";
exports.THUMBS_DOWN = "\ue6be";
exports.THUMBS_UP = "\ue6bd";
exports.TICK = "\u2713";
exports.TICK_CIRCLE = "\ue779";
exports.TIME = "\u23F2";
exports.TIMELINE_AREA_CHART = "\ue6cd";
exports.TIMELINE_BAR_CHART = "\ue620";
exports.TIMELINE_EVENTS = "\ue61e";
exports.TIMELINE_LINE_CHART = "\ue61f";
exports.TINT = "\ue6b2";
exports.TORCH = "\ue677";
exports.TRACTOR = "\ue90c";
exports.TRAIN = "\ue79f";
exports.TRANSLATE = "\ue759";
exports.TRASH = "\ue63b";
exports.TREE = "\ue7b7";
exports.TRENDING_DOWN = "\ue71a";
exports.TRENDING_UP = "\ue719";
exports.TRUCK = "\ue90b";
exports.TWO_COLUMNS = "\ue657";
exports.UNARCHIVE = "\ue906";
exports.UNDERLINE = "\u2381";
exports.UNDO = "\u238c";
exports.UNGROUP_OBJECTS = "\ue688";
exports.UNKNOWN_VEHICLE = "\ue73d";
exports.UNLOCK = "\ue626";
exports.UNPIN = "\ue650";
exports.UNRESOLVE = "\ue679";
exports.UPDATED = "\ue7a7";
exports.UPLOAD = "\ue68f";
exports.USER = "\ue627";
exports.VARIABLE = "\ue6f5";
exports.VERTICAL_BAR_CHART_ASC = "\ue75b";
exports.VERTICAL_BAR_CHART_DESC = "\ue71c";
exports.VERTICAL_DISTRIBUTION = "\ue721";
exports.VIDEO = "\ue6a0";
exports.VOLUME_DOWN = "\ue6a4";
exports.VOLUME_OFF = "\ue6a3";
exports.VOLUME_UP = "\ue6a5";
exports.WALK = "\ue79d";
exports.WARNING_SIGN = "\ue647";
exports.WATERFALL_CHART = "\ue6e6";
exports.WIDGET = "\ue678";
exports.WIDGET_BUTTON = "\ue790";
exports.WIDGET_FOOTER = "\ue792";
exports.WIDGET_HEADER = "\ue791";
exports.WRENCH = "\ue734";
exports.ZOOM_IN = "\ue641";
exports.ZOOM_OUT = "\ue642";
exports.ZOOM_TO_FIT = "\ue67b";
//# sourceMappingURL=iconContents.js.map