import React from "react";
interface Props {
    data: string;
    mediaType: "image/svg+xml";
}
export declare class SVG extends React.PureComponent<Props> {
    static defaultProps: {
        data: string;
        mediaType: string;
    };
    el?: HTMLElement | null;
    componentDidMount(): void;
    componentDidUpdate(): void;
    render(): JSX.Element;
}
export default SVG;
