declare const _default: "\n& a {\n  color: var(--link-color-unvisited, blue);\n}\n\n& a:visited {\n  color: var(--link-color-visited, blue);\n}\n\n& code {\n  font-family: \"Source Code Pro\";\n  white-space: pre-wrap;\n  font-size: 14px;\n}\n\n& pre {\n  white-space: pre-wrap;\n  font-size: 14px;\n  word-wrap: break-word;\n}\n\n& img {\n  display: block;\n  max-width: 100%;\n}\n\n& kbd {\n  display: inline-block;\n  border: 1px solid #ccc;\n  border-radius: 4px;\n  padding: 0.1em 0.5em;\n  margin: 0 0.2em;\n  box-shadow: 0 1px 0px rgba(0, 0, 0, 0.2), 0 0 0 2px #fff inset;\n  background-color: #f7f7f7;\n}\n\n& table {\n  border-collapse: collapse;\n}\n\n& th,\n& td,\n/* for legacy output handling */\n& .th,\n& .td {\n  padding: 0.5em 1em;\n  border: 1px solid var(--theme-app-border, #cbcbcb);\n}\n\n& th {\n  text-align: left;\n}\n\n& blockquote {\n  padding: 0.75em 0.5em 0.75em 1em;\n  background: var(--theme-cell-output-bg, white);\n  border-left: 0.5em solid #ddd;\n}\n\n& blockquote::before {\n  display: block;\n  height: 0;\n  content: \"“\";\n  margin-left: -0.95em;\n  font: italic 400%/1 Open Serif, Georgia, \"Times New Roman\", serif;\n  color: solid var(--theme-app-border, #cbcbcb);\n}\n\n/* for nested paragraphs in block quotes */\n& blockquote p {\n  display: inline;\n}\n\n& dd {\n  display: block;\n  -webkit-margin-start: 40px;\n}\n& dl {\n  display: block;\n  -webkit-margin-before: 1__qem;\n  -webkit-margin-after: 1em;\n  -webkit-margin-start: 0;\n  -webkit-margin-end: 0;\n}\n\n& dt {\n  display: block;\n}\n\n& dl {\n  width: 100%;\n  overflow: hidden;\n  padding: 0;\n  margin: 0;\n}\n\n& dt {\n  font-weight: bold;\n  float: left;\n  width: 20%;\n  /* adjust the width; make sure the total of both is 100% */\n  padding: 0;\n  margin: 0;\n}\n\n& dd {\n  float: left;\n  width: 80%;\n  /* adjust the width; make sure the total of both is 100% */\n  padding: 0;\n  margin: 0;\n}\n\n/** Adaptation for the R kernel's inline lists **/\n& .list-inline li {\n  display: inline;\n  padding-right: 20px;\n  text-align: center;\n}";
export default _default;
