/// <reference types="react" />
import { ImmutableErrorOutput } from "@nteract/commutable";
interface Props {
    className?: string;
    output: ImmutableErrorOutput;
    output_type: "error";
}
export declare const KernelOutputError: import("styled-components").StyledComponent<(props: Partial<Props>) => JSX.Element | null, any, {}, never>;
export default KernelOutputError;
