import React, { FC, HTMLAttributes } from "react";
import { DirectoryListItem } from "./sidebar-item";
export interface Props extends HTMLAttributes<HTMLDivElement> {
    isVisible?: boolean;
    subItems?: DirectoryListItem[];
    children?: React.ReactNode;
}
export declare const Sidebar: FC<Props>;
