export declare const BACKSPACE = 8;
export declare const TAB = 9;
export declare const ENTER = 13;
export declare const SHIFT = 16;
export declare const ESCAPE = 27;
export declare const SPACE = 32;
export declare const ARROW_LEFT = 37;
export declare const ARROW_UP = 38;
export declare const ARROW_RIGHT = 39;
export declare const ARROW_DOWN = 40;
export declare const DELETE = 46;
/** Returns whether the key code is `enter` or `space`, the two keys that can click a button. */
export declare function isKeyboardClick(keyCode: number): boolean;
