/*!
 * Copyright 2016 The ANTLR Project. All rights reserved.
 * Licensed under the BSD-3-Clause license. See LICENSE file in the project root for license information.
 */
export declare const enum TransitionType {
    EPSILON = 1,
    RANGE = 2,
    RULE = 3,
    PREDICATE = 4,
    ATOM = 5,
    ACTION = 6,
    SET = 7,
    NOT_SET = 8,
    WILDCARD = 9,
    PRECEDENCE = 10
}
