import { ArgumentArray } from './index.js';

declare namespace classNames {
	type Binding = Record<string, string>;
}

declare function classNames(this: classNames.Binding | undefined, ...args: ArgumentArray): string;

export = classNames;
