import * as React from "react";
interface InputProps {
    children: React.ReactNode;
    /**
     * Whether or not to render the children.
     */
    hidden?: boolean;
    className?: string;
}
declare class BareInput extends React.Component<InputProps> {
    static defaultProps: {
        children: null;
        hidden: boolean;
    };
    render(): JSX.Element | null;
}
export declare const Input: import("styled-components").StyledComponent<typeof BareInput, any, {}, never>;
export default Input;
