import PathProxy from '../../core/PathProxy';
export declare function buildPath(ctx: CanvasRenderingContext2D | PathProxy, shape: {
    x: number;
    y: number;
    width: number;
    height: number;
    r?: number | number[];
}): void;
