"use strict";
/*
 * Copyright 2021 Palantir Technologies, Inc. All rights reserved.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BARCODE = exports.BANK_ACCOUNT = exports.BAN_CIRCLE = exports.BADGE = exports.BACKLINK = exports.AUTOMATIC_UPDATES = exports.ASTERISK = exports.ARROWS_VERTICAL = exports.ARROWS_HORIZONTAL = exports.ARROW_UP = exports.ARROW_TOP_RIGHT = exports.ARROW_TOP_LEFT = exports.ARROW_RIGHT = exports.ARROW_LEFT = exports.ARROW_DOWN = exports.ARROW_BOTTOM_RIGHT = exports.ARROW_BOTTOM_LEFT = exports.ARRAY_TIMESTAMP = exports.ARRAY_STRING = exports.ARRAY_NUMERIC = exports.ARRAY_DATE = exports.ARRAY_BOOLEAN = exports.ARRAY = exports.AREA_OF_INTEREST = exports.ARCHIVE = exports.APPLICATIONS = exports.APPLICATION = exports.APP_HEADER = exports.ANTENNA = exports.ANNOTATION = exports.ALIGNMENT_VERTICAL_CENTER = exports.ALIGNMENT_TOP = exports.ALIGNMENT_RIGHT = exports.ALIGNMENT_LEFT = exports.ALIGNMENT_HORIZONTAL_CENTER = exports.ALIGNMENT_BOTTOM = exports.ALIGN_RIGHT = exports.ALIGN_LEFT = exports.ALIGN_JUSTIFY = exports.ALIGN_CENTER = exports.AIRPLANE = exports.ADD_TO_FOLDER = exports.ADD_TO_ARTIFACT = exports.ADD_ROW_TOP = exports.ADD_ROW_BOTTOM = exports.ADD_LOCATION = exports.ADD_COLUMN_RIGHT = exports.ADD_COLUMN_LEFT = exports.ADD_CLIP = exports.ADD = void 0;
exports.CONFIRM = exports.COMPRESSED = exports.COMPASS = exports.COMPARISON = exports.COMMENT = exports.COLUMN_LAYOUT = exports.COLLAPSE_ALL = exports.COG = exports.CODE_BLOCK = exports.CODE = exports.CLOUD_UPLOAD = exports.CLOUD_DOWNLOAD = exports.CLOUD = exports.CLIPBOARD = exports.CLIP = exports.CLEAN = exports.CITATION = exports.CIRCLE_ARROW_UP = exports.CIRCLE_ARROW_RIGHT = exports.CIRCLE_ARROW_LEFT = exports.CIRCLE_ARROW_DOWN = exports.CIRCLE = exports.CHEVRON_UP = exports.CHEVRON_RIGHT = exports.CHEVRON_LEFT = exports.CHEVRON_FORWARD = exports.CHEVRON_DOWN = exports.CHEVRON_BACKWARD = exports.CHAT = exports.CHART = exports.CHANGES = exports.CELL_TOWER = exports.CARGO_SHIP = exports.CARET_UP = exports.CARET_RIGHT = exports.CARET_LEFT = exports.CARET_DOWN = exports.CAMERA = exports.CALENDAR = exports.CALCULATOR = exports.BUILD = exports.BUGGY = exports.BRING_DATA = exports.BRIEFCASE = exports.BOX = exports.BOOKMARK = exports.BOOK = exports.BOLD = exports.BLOCKED_PERSON = exports.BLANK = void 0;
exports.EMOJI = exports.EJECT = exports.EDIT = exports.DUPLICATE = exports.DRIVE_TIME = exports.DRAWER_RIGHT_FILLED = exports.DRAWER_RIGHT = exports.DRAWER_LEFT_FILLED = exports.DRAWER_LEFT = exports.DRAW = exports.DRAG_HANDLE_VERTICAL = exports.DRAG_HANDLE_HORIZONTAL = exports.DOWNLOAD = exports.DOUGHNUT_CHART = exports.DOUBLE_CHEVRON_UP = exports.DOUBLE_CHEVRON_RIGHT = exports.DOUBLE_CHEVRON_LEFT = exports.DOUBLE_CHEVRON_DOWN = exports.DOUBLE_CARET_VERTICAL = exports.DOUBLE_CARET_HORIZONTAL = exports.DOT = exports.DOLLAR = exports.DOCUMENT_SHARE = exports.DOCUMENT_OPEN = exports.DOCUMENT = exports.DISABLE = exports.DIRECTION_RIGHT = exports.DIRECTION_LEFT = exports.DIAGRAM_TREE = exports.DIAGNOSIS = exports.DESKTOP = exports.DERIVE_COLUMN = exports.DELTA = exports.DELETE = exports.DATABASE = exports.DATA_LINEAGE = exports.DATA_CONNECTION = exports.DASHBOARD = exports.CYCLE = exports.CUT = exports.CURVED_RANGE_CHART = exports.CUBE_REMOVE = exports.CUBE_ADD = exports.CUBE = exports.CROWN = exports.CROSS = exports.CREDIT_CARD = exports.CONTROL = exports.CONTRAST = exports.CONSOLE = void 0;
exports.GEOLOCATION = exports.GEOFENCE = exports.GANTT_CHART = exports.FUNCTION = exports.FULLSCREEN = exports.FULL_STACKED_CHART = exports.FULL_CIRCLE = exports.FORM = exports.FORK = exports.FONT = exports.FOLLOWING = exports.FOLLOWER = exports.FOLDER_SHARED_OPEN = exports.FOLDER_SHARED = exports.FOLDER_OPEN = exports.FOLDER_NEW = exports.FOLDER_CLOSE = exports.FLOWS = exports.FLOW_REVIEW_BRANCH = exports.FLOW_REVIEW = exports.FLOW_LINEAR = exports.FLOW_END = exports.FLOW_BRANCH = exports.FLOPPY_DISK = exports.FLASH = exports.FLAME = exports.FLAG = exports.FILTER_REMOVE = exports.FILTER_OPEN = exports.FILTER_LIST = exports.FILTER_KEEP = exports.FILTER = exports.FILM = exports.FEED_SUBSCRIBED = exports.FEED = exports.FAST_FORWARD = exports.FAST_BACKWARD = exports.EYE_OPEN = exports.EYE_ON = exports.EYE_OFF = exports.EXPORT = exports.EXPAND_ALL = exports.EXCLUDE_ROW = exports.EXCHANGE = exports.EURO = exports.ERROR = exports.ERASER = exports.EQUALS = exports.ENVELOPE = exports.ENDORSED = void 0;
exports.IMPORT = exports.IMAGE_ROTATE_RIGHT = exports.IMAGE_ROTATE_LEFT = exports.ID_NUMBER = exports.HURRICANE = exports.HORIZONTAL_DISTRIBUTION = exports.HORIZONTAL_BAR_CHART_DESC = exports.HORIZONTAL_BAR_CHART_ASC = exports.HORIZONTAL_BAR_CHART = exports.HOME = exports.HISTORY = exports.HIGHLIGHT = exports.HIGH_PRIORITY = exports.HELPER_MANAGEMENT = exports.HELP = exports.HELICOPTER = exports.HEATMAP = exports.HEAT_GRID = exports.HEART_BROKEN = exports.HEART = exports.HEADSET = exports.HEADER_TWO = exports.HEADER_THREE = exports.HEADER_ONE = exports.HEADER = exports.HAT = exports.HAND_UP = exports.HAND_RIGHT = exports.HAND_LEFT = exports.HAND_DOWN = exports.HAND = exports.GROUPED_BAR_CHART = exports.GROUP_OBJECTS = exports.GRID_VIEW = exports.GRID = exports.GREATER_THAN_OR_EQUAL_TO = exports.GREATER_THAN = exports.GRAPH_REMOVE = exports.GRAPH = exports.GLOBE_NETWORK = exports.GLOBE = exports.GLASS = exports.GIT_REPO = exports.GIT_PUSH = exports.GIT_PULL = exports.GIT_NEW_BRANCH = exports.GIT_MERGE = exports.GIT_COMMIT = exports.GIT_BRANCH = exports.GEOSEARCH = void 0;
exports.LIGHTNING = exports.LIGHTBULB = exports.LIFESAVER = exports.LESS_THAN_OR_EQUAL_TO = exports.LESS_THAN = exports.LEFT_JOIN = exports.LEARNING = exports.LAYOUT_SORTED_CLUSTERS = exports.LAYOUT_SKEW_GRID = exports.LAYOUT_LINEAR = exports.LAYOUT_HIERARCHY = exports.LAYOUT_GROUP_BY = exports.LAYOUT_GRID = exports.LAYOUT_CIRCLE = exports.LAYOUT_BALLOON = exports.LAYOUT_AUTO = exports.LAYOUT = exports.LAYERS = exports.LAYER_OUTLINE = exports.LAYER = exports.LABEL = exports.LAB_TEST = exports.KNOWN_VEHICLE = exports.KEY_TAB = exports.KEY_SHIFT = exports.KEY_OPTION = exports.KEY_ESCAPE = exports.KEY_ENTER = exports.KEY_DELETE = exports.KEY_CONTROL = exports.KEY_COMMAND = exports.KEY_BACKSPACE = exports.KEY = exports.JOIN_TABLE = exports.ITALIC = exports.ISSUE_NEW = exports.ISSUE_CLOSED = exports.ISSUE = exports.IP_ADDRESS = exports.INTERSECTION = exports.INSERT = exports.INNER_JOIN = exports.INHERITED_GROUP = exports.INHERITANCE = exports.INFO_SIGN = exports.INBOX_UPDATE = exports.INBOX_SEARCH = exports.INBOX_GEO = exports.INBOX_FILTERED = exports.INBOX = void 0;
exports.NOTIFICATIONS_UPDATED = exports.NOTIFICATIONS_SNOOZE = exports.NOTIFICATIONS = exports.NOT_EQUAL_TO = exports.NINJA = exports.NEW_TEXT_BOX = exports.NEW_PRESCRIPTION = exports.NEW_PERSON = exports.NEW_OBJECT = exports.NEW_LINK = exports.NEW_LAYERS = exports.NEW_LAYER = exports.NEW_GRID_ITEM = exports.NEW_DRAWING = exports.NEST = exports.MUSIC = exports.MULTI_SELECT = exports.MUGSHOT = exports.MOVE = exports.MOUNTAIN = exports.MORE = exports.MOON = exports.MODAL_FILLED = exports.MODAL = exports.MOBILE_VIDEO = exports.MOBILE_PHONE = exports.MINUS = exports.MINIMIZE = exports.MERGE_LINKS = exports.MERGE_COLUMNS = exports.MENU_OPEN = exports.MENU_CLOSED = exports.MENU = exports.MEDIA = exports.MAXIMIZE = exports.MAP_MARKER = exports.MAP_CREATE = exports.MAP = exports.MANY_TO_ONE = exports.MANY_TO_MANY = exports.MANUALLY_ENTERED_DATA = exports.MANUAL = exports.LOG_OUT = exports.LOG_IN = exports.LOCK = exports.LOCATE = exports.LIST_DETAIL_VIEW = exports.LIST_COLUMNS = exports.LIST = exports.LINK = void 0;
exports.REMOVE_ROW_BOTTOM = exports.REMOVE_COLUMN_RIGHT = exports.REMOVE_COLUMN_LEFT = exports.REMOVE_COLUMN = exports.REMOVE = exports.REGRESSION_CHART = exports.REFRESH = exports.REDO = exports.RECTANGLE = exports.RECORD = exports.RANDOM = exports.RAIN = exports.PULSE = exports.PUBLISH_FUNCTION = exports.PROPERTY = exports.PROPERTIES = exports.PROJECTS = exports.PRINT = exports.PRESENTATION = exports.PRESCRIPTION = exports.PREDICTIVE_ANALYSIS = exports.POWER = exports.POLYGON_FILTER = exports.PLUS = exports.PLAY = exports.PIVOT_TABLE = exports.PIVOT = exports.PIN = exports.PIE_CHART = exports.PHONE = exports.PERSON = exports.PERCENTAGE = exports.PEOPLE = exports.PAUSE = exports.PATH_SEARCH = exports.PATH = exports.PARAGRAPH = exports.PAPERCLIP = exports.PANEL_TABLE = exports.PANEL_STATS = exports.PAGE_LAYOUT = exports.OUTDATED = exports.ONE_TO_ONE = exports.ONE_TO_MANY = exports.ONE_COLUMN = exports.OIL_FIELD = exports.OFFLINE = exports.OFFICE = exports.NUMERICAL = exports.NUMBERED_LIST = void 0;
exports.SORT = exports.SOCIAL_MEDIA = exports.SNOWFLAKE = exports.SMALL_TICK = exports.SMALL_SQUARE = exports.SMALL_PLUS = exports.SMALL_MINUS = exports.SMALL_CROSS = exports.SLASH = exports.SIM_CARD = exports.SIGNAL_SEARCH = exports.SHOPPING_CART = exports.SHOP = exports.SHIP = exports.SHIELD = exports.SHARED_FILTER = exports.SHARE = exports.SHAPES = exports.SETTINGS = exports.SERIES_SEARCH = exports.SERIES_FILTERED = exports.SERIES_DERIVED = exports.SERIES_CONFIGURATION = exports.SERIES_ADD = exports.SEND_TO_MAP = exports.SEND_TO_GRAPH = exports.SEND_TO = exports.SEND_MESSAGE = exports.SELECTION = exports.SELECT = exports.SEGMENTED_CONTROL = exports.SEARCH_TEXT = exports.SEARCH_TEMPLATE = exports.SEARCH_AROUND = exports.SEARCH = exports.SCATTER_PLOT = exports.SAVED = exports.SATELLITE = exports.ROUTE = exports.ROTATE_PAGE = exports.ROTATE_DOCUMENT = exports.ROCKET_SLANT = exports.ROCKET = exports.RING = exports.RIGHT_JOIN = exports.RIG = exports.RESOLVE = exports.RESET = exports.REPEAT = exports.REMOVE_ROW_TOP = void 0;
exports.TIMELINE_BAR_CHART = exports.TIMELINE_AREA_CHART = exports.TIME = exports.TICK_CIRCLE = exports.TICK = exports.THUMBS_UP = exports.THUMBS_DOWN = exports.THIRD_PARTY = exports.TH_LIST = exports.TH_FILTERED = exports.TH_DISCONNECT = exports.TH_DERIVED = exports.TH = exports.TEXT_HIGHLIGHT = exports.TEMPERATURE = exports.TAXI = exports.TARGET = exports.TANK = exports.TAKE_ACTION = exports.TAG = exports.SYRINGE = exports.SYMBOL_TRIANGLE_UP = exports.SYMBOL_TRIANGLE_DOWN = exports.SYMBOL_SQUARE = exports.SYMBOL_RECTANGLE = exports.SYMBOL_DIAMOND = exports.SYMBOL_CROSS = exports.SYMBOL_CIRCLE = exports.SWITCH = exports.SWAP_VERTICAL = exports.SWAP_HORIZONTAL = exports.STYLE = exports.STRIKETHROUGH = exports.STOPWATCH = exports.STOP = exports.STEP_FORWARD = exports.STEP_CHART = exports.STEP_BACKWARD = exports.STAR_EMPTY = exports.STAR = exports.STADIUM_GEOMETRY = exports.STACKED_CHART = exports.SQUARE = exports.SPLIT_COLUMNS = exports.SORT_NUMERICAL_DESC = exports.SORT_NUMERICAL = exports.SORT_DESC = exports.SORT_ASC = exports.SORT_ALPHABETICAL_DESC = exports.SORT_ALPHABETICAL = void 0;
exports.ZOOM_TO_FIT = exports.ZOOM_OUT = exports.ZOOM_IN = exports.WRENCH = exports.WIND = exports.WIDGET_HEADER = exports.WIDGET_FOOTER = exports.WIDGET_BUTTON = exports.WIDGET = exports.WAVES = exports.WATERFALL_CHART = exports.WARNING_SIGN = exports.WALK = exports.VOLUME_UP = exports.VOLUME_OFF = exports.VOLUME_DOWN = exports.VIRUS = exports.VIDEO = exports.VERTICAL_DISTRIBUTION = exports.VERTICAL_BAR_CHART_DESC = exports.VERTICAL_BAR_CHART_ASC = exports.VARIABLE = exports.USER = exports.UPLOAD = exports.UPDATED = exports.UNRESOLVE = exports.UNPIN = exports.UNLOCK = exports.UNKNOWN_VEHICLE = exports.UNGROUP_OBJECTS = exports.UNDO = exports.UNDERLINE = exports.UNARCHIVE = exports.TWO_COLUMNS = exports.TRUCK = exports.TRENDING_UP = exports.TRENDING_DOWN = exports.TREE = exports.TRASH = exports.TRANSLATE = exports.TRAIN = exports.TRACTOR = exports.TORCH = exports.TINT = exports.TIMELINE_LINE_CHART = exports.TIMELINE_EVENTS = void 0;
exports.ADD = "add";
exports.ADD_CLIP = "add-clip";
exports.ADD_COLUMN_LEFT = "add-column-left";
exports.ADD_COLUMN_RIGHT = "add-column-right";
exports.ADD_LOCATION = "add-location";
exports.ADD_ROW_BOTTOM = "add-row-bottom";
exports.ADD_ROW_TOP = "add-row-top";
exports.ADD_TO_ARTIFACT = "add-to-artifact";
exports.ADD_TO_FOLDER = "add-to-folder";
exports.AIRPLANE = "airplane";
exports.ALIGN_CENTER = "align-center";
exports.ALIGN_JUSTIFY = "align-justify";
exports.ALIGN_LEFT = "align-left";
exports.ALIGN_RIGHT = "align-right";
exports.ALIGNMENT_BOTTOM = "alignment-bottom";
exports.ALIGNMENT_HORIZONTAL_CENTER = "alignment-horizontal-center";
exports.ALIGNMENT_LEFT = "alignment-left";
exports.ALIGNMENT_RIGHT = "alignment-right";
exports.ALIGNMENT_TOP = "alignment-top";
exports.ALIGNMENT_VERTICAL_CENTER = "alignment-vertical-center";
exports.ANNOTATION = "annotation";
exports.ANTENNA = "antenna";
exports.APP_HEADER = "app-header";
exports.APPLICATION = "application";
exports.APPLICATIONS = "applications";
exports.ARCHIVE = "archive";
exports.AREA_OF_INTEREST = "area-of-interest";
exports.ARRAY = "array";
exports.ARRAY_BOOLEAN = "array-boolean";
exports.ARRAY_DATE = "array-date";
exports.ARRAY_NUMERIC = "array-numeric";
exports.ARRAY_STRING = "array-string";
exports.ARRAY_TIMESTAMP = "array-timestamp";
exports.ARROW_BOTTOM_LEFT = "arrow-bottom-left";
exports.ARROW_BOTTOM_RIGHT = "arrow-bottom-right";
exports.ARROW_DOWN = "arrow-down";
exports.ARROW_LEFT = "arrow-left";
exports.ARROW_RIGHT = "arrow-right";
exports.ARROW_TOP_LEFT = "arrow-top-left";
exports.ARROW_TOP_RIGHT = "arrow-top-right";
exports.ARROW_UP = "arrow-up";
exports.ARROWS_HORIZONTAL = "arrows-horizontal";
exports.ARROWS_VERTICAL = "arrows-vertical";
exports.ASTERISK = "asterisk";
exports.AUTOMATIC_UPDATES = "automatic-updates";
exports.BACKLINK = "backlink";
exports.BADGE = "badge";
exports.BAN_CIRCLE = "ban-circle";
exports.BANK_ACCOUNT = "bank-account";
exports.BARCODE = "barcode";
exports.BLANK = "blank";
exports.BLOCKED_PERSON = "blocked-person";
exports.BOLD = "bold";
exports.BOOK = "book";
exports.BOOKMARK = "bookmark";
exports.BOX = "box";
exports.BRIEFCASE = "briefcase";
exports.BRING_DATA = "bring-data";
exports.BUGGY = "buggy";
exports.BUILD = "build";
exports.CALCULATOR = "calculator";
exports.CALENDAR = "calendar";
exports.CAMERA = "camera";
exports.CARET_DOWN = "caret-down";
exports.CARET_LEFT = "caret-left";
exports.CARET_RIGHT = "caret-right";
exports.CARET_UP = "caret-up";
exports.CARGO_SHIP = "cargo-ship";
exports.CELL_TOWER = "cell-tower";
exports.CHANGES = "changes";
exports.CHART = "chart";
exports.CHAT = "chat";
exports.CHEVRON_BACKWARD = "chevron-backward";
exports.CHEVRON_DOWN = "chevron-down";
exports.CHEVRON_FORWARD = "chevron-forward";
exports.CHEVRON_LEFT = "chevron-left";
exports.CHEVRON_RIGHT = "chevron-right";
exports.CHEVRON_UP = "chevron-up";
exports.CIRCLE = "circle";
exports.CIRCLE_ARROW_DOWN = "circle-arrow-down";
exports.CIRCLE_ARROW_LEFT = "circle-arrow-left";
exports.CIRCLE_ARROW_RIGHT = "circle-arrow-right";
exports.CIRCLE_ARROW_UP = "circle-arrow-up";
exports.CITATION = "citation";
exports.CLEAN = "clean";
exports.CLIP = "clip";
exports.CLIPBOARD = "clipboard";
exports.CLOUD = "cloud";
exports.CLOUD_DOWNLOAD = "cloud-download";
exports.CLOUD_UPLOAD = "cloud-upload";
exports.CODE = "code";
exports.CODE_BLOCK = "code-block";
exports.COG = "cog";
exports.COLLAPSE_ALL = "collapse-all";
exports.COLUMN_LAYOUT = "column-layout";
exports.COMMENT = "comment";
exports.COMPARISON = "comparison";
exports.COMPASS = "compass";
exports.COMPRESSED = "compressed";
exports.CONFIRM = "confirm";
exports.CONSOLE = "console";
exports.CONTRAST = "contrast";
exports.CONTROL = "control";
exports.CREDIT_CARD = "credit-card";
exports.CROSS = "cross";
exports.CROWN = "crown";
exports.CUBE = "cube";
exports.CUBE_ADD = "cube-add";
exports.CUBE_REMOVE = "cube-remove";
exports.CURVED_RANGE_CHART = "curved-range-chart";
exports.CUT = "cut";
exports.CYCLE = "cycle";
exports.DASHBOARD = "dashboard";
exports.DATA_CONNECTION = "data-connection";
exports.DATA_LINEAGE = "data-lineage";
exports.DATABASE = "database";
exports.DELETE = "delete";
exports.DELTA = "delta";
exports.DERIVE_COLUMN = "derive-column";
exports.DESKTOP = "desktop";
exports.DIAGNOSIS = "diagnosis";
exports.DIAGRAM_TREE = "diagram-tree";
exports.DIRECTION_LEFT = "direction-left";
exports.DIRECTION_RIGHT = "direction-right";
exports.DISABLE = "disable";
exports.DOCUMENT = "document";
exports.DOCUMENT_OPEN = "document-open";
exports.DOCUMENT_SHARE = "document-share";
exports.DOLLAR = "dollar";
exports.DOT = "dot";
exports.DOUBLE_CARET_HORIZONTAL = "double-caret-horizontal";
exports.DOUBLE_CARET_VERTICAL = "double-caret-vertical";
exports.DOUBLE_CHEVRON_DOWN = "double-chevron-down";
exports.DOUBLE_CHEVRON_LEFT = "double-chevron-left";
exports.DOUBLE_CHEVRON_RIGHT = "double-chevron-right";
exports.DOUBLE_CHEVRON_UP = "double-chevron-up";
exports.DOUGHNUT_CHART = "doughnut-chart";
exports.DOWNLOAD = "download";
exports.DRAG_HANDLE_HORIZONTAL = "drag-handle-horizontal";
exports.DRAG_HANDLE_VERTICAL = "drag-handle-vertical";
exports.DRAW = "draw";
exports.DRAWER_LEFT = "drawer-left";
exports.DRAWER_LEFT_FILLED = "drawer-left-filled";
exports.DRAWER_RIGHT = "drawer-right";
exports.DRAWER_RIGHT_FILLED = "drawer-right-filled";
exports.DRIVE_TIME = "drive-time";
exports.DUPLICATE = "duplicate";
exports.EDIT = "edit";
exports.EJECT = "eject";
exports.EMOJI = "emoji";
exports.ENDORSED = "endorsed";
exports.ENVELOPE = "envelope";
exports.EQUALS = "equals";
exports.ERASER = "eraser";
exports.ERROR = "error";
exports.EURO = "euro";
exports.EXCHANGE = "exchange";
exports.EXCLUDE_ROW = "exclude-row";
exports.EXPAND_ALL = "expand-all";
exports.EXPORT = "export";
exports.EYE_OFF = "eye-off";
exports.EYE_ON = "eye-on";
exports.EYE_OPEN = "eye-open";
exports.FAST_BACKWARD = "fast-backward";
exports.FAST_FORWARD = "fast-forward";
exports.FEED = "feed";
exports.FEED_SUBSCRIBED = "feed-subscribed";
exports.FILM = "film";
exports.FILTER = "filter";
exports.FILTER_KEEP = "filter-keep";
exports.FILTER_LIST = "filter-list";
exports.FILTER_OPEN = "filter-open";
exports.FILTER_REMOVE = "filter-remove";
exports.FLAG = "flag";
exports.FLAME = "flame";
exports.FLASH = "flash";
exports.FLOPPY_DISK = "floppy-disk";
exports.FLOW_BRANCH = "flow-branch";
exports.FLOW_END = "flow-end";
exports.FLOW_LINEAR = "flow-linear";
exports.FLOW_REVIEW = "flow-review";
exports.FLOW_REVIEW_BRANCH = "flow-review-branch";
exports.FLOWS = "flows";
exports.FOLDER_CLOSE = "folder-close";
exports.FOLDER_NEW = "folder-new";
exports.FOLDER_OPEN = "folder-open";
exports.FOLDER_SHARED = "folder-shared";
exports.FOLDER_SHARED_OPEN = "folder-shared-open";
exports.FOLLOWER = "follower";
exports.FOLLOWING = "following";
exports.FONT = "font";
exports.FORK = "fork";
exports.FORM = "form";
exports.FULL_CIRCLE = "full-circle";
exports.FULL_STACKED_CHART = "full-stacked-chart";
exports.FULLSCREEN = "fullscreen";
exports.FUNCTION = "function";
exports.GANTT_CHART = "gantt-chart";
exports.GEOFENCE = "geofence";
exports.GEOLOCATION = "geolocation";
exports.GEOSEARCH = "geosearch";
exports.GIT_BRANCH = "git-branch";
exports.GIT_COMMIT = "git-commit";
exports.GIT_MERGE = "git-merge";
exports.GIT_NEW_BRANCH = "git-new-branch";
exports.GIT_PULL = "git-pull";
exports.GIT_PUSH = "git-push";
exports.GIT_REPO = "git-repo";
exports.GLASS = "glass";
exports.GLOBE = "globe";
exports.GLOBE_NETWORK = "globe-network";
exports.GRAPH = "graph";
exports.GRAPH_REMOVE = "graph-remove";
exports.GREATER_THAN = "greater-than";
exports.GREATER_THAN_OR_EQUAL_TO = "greater-than-or-equal-to";
exports.GRID = "grid";
exports.GRID_VIEW = "grid-view";
exports.GROUP_OBJECTS = "group-objects";
exports.GROUPED_BAR_CHART = "grouped-bar-chart";
exports.HAND = "hand";
exports.HAND_DOWN = "hand-down";
exports.HAND_LEFT = "hand-left";
exports.HAND_RIGHT = "hand-right";
exports.HAND_UP = "hand-up";
exports.HAT = "hat";
exports.HEADER = "header";
exports.HEADER_ONE = "header-one";
exports.HEADER_THREE = "header-three";
exports.HEADER_TWO = "header-two";
exports.HEADSET = "headset";
exports.HEART = "heart";
exports.HEART_BROKEN = "heart-broken";
exports.HEAT_GRID = "heat-grid";
exports.HEATMAP = "heatmap";
exports.HELICOPTER = "helicopter";
exports.HELP = "help";
exports.HELPER_MANAGEMENT = "helper-management";
exports.HIGH_PRIORITY = "high-priority";
exports.HIGHLIGHT = "highlight";
exports.HISTORY = "history";
exports.HOME = "home";
exports.HORIZONTAL_BAR_CHART = "horizontal-bar-chart";
exports.HORIZONTAL_BAR_CHART_ASC = "horizontal-bar-chart-asc";
exports.HORIZONTAL_BAR_CHART_DESC = "horizontal-bar-chart-desc";
exports.HORIZONTAL_DISTRIBUTION = "horizontal-distribution";
exports.HURRICANE = "hurricane";
exports.ID_NUMBER = "id-number";
exports.IMAGE_ROTATE_LEFT = "image-rotate-left";
exports.IMAGE_ROTATE_RIGHT = "image-rotate-right";
exports.IMPORT = "import";
exports.INBOX = "inbox";
exports.INBOX_FILTERED = "inbox-filtered";
exports.INBOX_GEO = "inbox-geo";
exports.INBOX_SEARCH = "inbox-search";
exports.INBOX_UPDATE = "inbox-update";
exports.INFO_SIGN = "info-sign";
exports.INHERITANCE = "inheritance";
exports.INHERITED_GROUP = "inherited-group";
exports.INNER_JOIN = "inner-join";
exports.INSERT = "insert";
exports.INTERSECTION = "intersection";
exports.IP_ADDRESS = "ip-address";
exports.ISSUE = "issue";
exports.ISSUE_CLOSED = "issue-closed";
exports.ISSUE_NEW = "issue-new";
exports.ITALIC = "italic";
exports.JOIN_TABLE = "join-table";
exports.KEY = "key";
exports.KEY_BACKSPACE = "key-backspace";
exports.KEY_COMMAND = "key-command";
exports.KEY_CONTROL = "key-control";
exports.KEY_DELETE = "key-delete";
exports.KEY_ENTER = "key-enter";
exports.KEY_ESCAPE = "key-escape";
exports.KEY_OPTION = "key-option";
exports.KEY_SHIFT = "key-shift";
exports.KEY_TAB = "key-tab";
exports.KNOWN_VEHICLE = "known-vehicle";
exports.LAB_TEST = "lab-test";
exports.LABEL = "label";
exports.LAYER = "layer";
exports.LAYER_OUTLINE = "layer-outline";
exports.LAYERS = "layers";
exports.LAYOUT = "layout";
exports.LAYOUT_AUTO = "layout-auto";
exports.LAYOUT_BALLOON = "layout-balloon";
exports.LAYOUT_CIRCLE = "layout-circle";
exports.LAYOUT_GRID = "layout-grid";
exports.LAYOUT_GROUP_BY = "layout-group-by";
exports.LAYOUT_HIERARCHY = "layout-hierarchy";
exports.LAYOUT_LINEAR = "layout-linear";
exports.LAYOUT_SKEW_GRID = "layout-skew-grid";
exports.LAYOUT_SORTED_CLUSTERS = "layout-sorted-clusters";
exports.LEARNING = "learning";
exports.LEFT_JOIN = "left-join";
exports.LESS_THAN = "less-than";
exports.LESS_THAN_OR_EQUAL_TO = "less-than-or-equal-to";
exports.LIFESAVER = "lifesaver";
exports.LIGHTBULB = "lightbulb";
exports.LIGHTNING = "lightning";
exports.LINK = "link";
exports.LIST = "list";
exports.LIST_COLUMNS = "list-columns";
exports.LIST_DETAIL_VIEW = "list-detail-view";
exports.LOCATE = "locate";
exports.LOCK = "lock";
exports.LOG_IN = "log-in";
exports.LOG_OUT = "log-out";
exports.MANUAL = "manual";
exports.MANUALLY_ENTERED_DATA = "manually-entered-data";
exports.MANY_TO_MANY = "many-to-many";
exports.MANY_TO_ONE = "many-to-one";
exports.MAP = "map";
exports.MAP_CREATE = "map-create";
exports.MAP_MARKER = "map-marker";
exports.MAXIMIZE = "maximize";
exports.MEDIA = "media";
exports.MENU = "menu";
exports.MENU_CLOSED = "menu-closed";
exports.MENU_OPEN = "menu-open";
exports.MERGE_COLUMNS = "merge-columns";
exports.MERGE_LINKS = "merge-links";
exports.MINIMIZE = "minimize";
exports.MINUS = "minus";
exports.MOBILE_PHONE = "mobile-phone";
exports.MOBILE_VIDEO = "mobile-video";
exports.MODAL = "modal";
exports.MODAL_FILLED = "modal-filled";
exports.MOON = "moon";
exports.MORE = "more";
exports.MOUNTAIN = "mountain";
exports.MOVE = "move";
exports.MUGSHOT = "mugshot";
exports.MULTI_SELECT = "multi-select";
exports.MUSIC = "music";
exports.NEST = "nest";
exports.NEW_DRAWING = "new-drawing";
exports.NEW_GRID_ITEM = "new-grid-item";
exports.NEW_LAYER = "new-layer";
exports.NEW_LAYERS = "new-layers";
exports.NEW_LINK = "new-link";
exports.NEW_OBJECT = "new-object";
exports.NEW_PERSON = "new-person";
exports.NEW_PRESCRIPTION = "new-prescription";
exports.NEW_TEXT_BOX = "new-text-box";
exports.NINJA = "ninja";
exports.NOT_EQUAL_TO = "not-equal-to";
exports.NOTIFICATIONS = "notifications";
exports.NOTIFICATIONS_SNOOZE = "notifications-snooze";
exports.NOTIFICATIONS_UPDATED = "notifications-updated";
exports.NUMBERED_LIST = "numbered-list";
exports.NUMERICAL = "numerical";
exports.OFFICE = "office";
exports.OFFLINE = "offline";
exports.OIL_FIELD = "oil-field";
exports.ONE_COLUMN = "one-column";
exports.ONE_TO_MANY = "one-to-many";
exports.ONE_TO_ONE = "one-to-one";
exports.OUTDATED = "outdated";
exports.PAGE_LAYOUT = "page-layout";
exports.PANEL_STATS = "panel-stats";
exports.PANEL_TABLE = "panel-table";
exports.PAPERCLIP = "paperclip";
exports.PARAGRAPH = "paragraph";
exports.PATH = "path";
exports.PATH_SEARCH = "path-search";
exports.PAUSE = "pause";
exports.PEOPLE = "people";
exports.PERCENTAGE = "percentage";
exports.PERSON = "person";
exports.PHONE = "phone";
exports.PIE_CHART = "pie-chart";
exports.PIN = "pin";
exports.PIVOT = "pivot";
exports.PIVOT_TABLE = "pivot-table";
exports.PLAY = "play";
exports.PLUS = "plus";
exports.POLYGON_FILTER = "polygon-filter";
exports.POWER = "power";
exports.PREDICTIVE_ANALYSIS = "predictive-analysis";
exports.PRESCRIPTION = "prescription";
exports.PRESENTATION = "presentation";
exports.PRINT = "print";
exports.PROJECTS = "projects";
exports.PROPERTIES = "properties";
exports.PROPERTY = "property";
exports.PUBLISH_FUNCTION = "publish-function";
exports.PULSE = "pulse";
exports.RAIN = "rain";
exports.RANDOM = "random";
exports.RECORD = "record";
exports.RECTANGLE = "rectangle";
exports.REDO = "redo";
exports.REFRESH = "refresh";
exports.REGRESSION_CHART = "regression-chart";
exports.REMOVE = "remove";
exports.REMOVE_COLUMN = "remove-column";
exports.REMOVE_COLUMN_LEFT = "remove-column-left";
exports.REMOVE_COLUMN_RIGHT = "remove-column-right";
exports.REMOVE_ROW_BOTTOM = "remove-row-bottom";
exports.REMOVE_ROW_TOP = "remove-row-top";
exports.REPEAT = "repeat";
exports.RESET = "reset";
exports.RESOLVE = "resolve";
exports.RIG = "rig";
exports.RIGHT_JOIN = "right-join";
exports.RING = "ring";
exports.ROCKET = "rocket";
exports.ROCKET_SLANT = "rocket-slant";
exports.ROTATE_DOCUMENT = "rotate-document";
exports.ROTATE_PAGE = "rotate-page";
exports.ROUTE = "route";
exports.SATELLITE = "satellite";
exports.SAVED = "saved";
exports.SCATTER_PLOT = "scatter-plot";
exports.SEARCH = "search";
exports.SEARCH_AROUND = "search-around";
exports.SEARCH_TEMPLATE = "search-template";
exports.SEARCH_TEXT = "search-text";
exports.SEGMENTED_CONTROL = "segmented-control";
exports.SELECT = "select";
exports.SELECTION = "selection";
exports.SEND_MESSAGE = "send-message";
exports.SEND_TO = "send-to";
exports.SEND_TO_GRAPH = "send-to-graph";
exports.SEND_TO_MAP = "send-to-map";
exports.SERIES_ADD = "series-add";
exports.SERIES_CONFIGURATION = "series-configuration";
exports.SERIES_DERIVED = "series-derived";
exports.SERIES_FILTERED = "series-filtered";
exports.SERIES_SEARCH = "series-search";
exports.SETTINGS = "settings";
exports.SHAPES = "shapes";
exports.SHARE = "share";
exports.SHARED_FILTER = "shared-filter";
exports.SHIELD = "shield";
exports.SHIP = "ship";
exports.SHOP = "shop";
exports.SHOPPING_CART = "shopping-cart";
exports.SIGNAL_SEARCH = "signal-search";
exports.SIM_CARD = "sim-card";
exports.SLASH = "slash";
exports.SMALL_CROSS = "small-cross";
exports.SMALL_MINUS = "small-minus";
exports.SMALL_PLUS = "small-plus";
exports.SMALL_SQUARE = "small-square";
exports.SMALL_TICK = "small-tick";
exports.SNOWFLAKE = "snowflake";
exports.SOCIAL_MEDIA = "social-media";
exports.SORT = "sort";
exports.SORT_ALPHABETICAL = "sort-alphabetical";
exports.SORT_ALPHABETICAL_DESC = "sort-alphabetical-desc";
exports.SORT_ASC = "sort-asc";
exports.SORT_DESC = "sort-desc";
exports.SORT_NUMERICAL = "sort-numerical";
exports.SORT_NUMERICAL_DESC = "sort-numerical-desc";
exports.SPLIT_COLUMNS = "split-columns";
exports.SQUARE = "square";
exports.STACKED_CHART = "stacked-chart";
exports.STADIUM_GEOMETRY = "stadium-geometry";
exports.STAR = "star";
exports.STAR_EMPTY = "star-empty";
exports.STEP_BACKWARD = "step-backward";
exports.STEP_CHART = "step-chart";
exports.STEP_FORWARD = "step-forward";
exports.STOP = "stop";
exports.STOPWATCH = "stopwatch";
exports.STRIKETHROUGH = "strikethrough";
exports.STYLE = "style";
exports.SWAP_HORIZONTAL = "swap-horizontal";
exports.SWAP_VERTICAL = "swap-vertical";
exports.SWITCH = "switch";
exports.SYMBOL_CIRCLE = "symbol-circle";
exports.SYMBOL_CROSS = "symbol-cross";
exports.SYMBOL_DIAMOND = "symbol-diamond";
exports.SYMBOL_RECTANGLE = "symbol-rectangle";
exports.SYMBOL_SQUARE = "symbol-square";
exports.SYMBOL_TRIANGLE_DOWN = "symbol-triangle-down";
exports.SYMBOL_TRIANGLE_UP = "symbol-triangle-up";
exports.SYRINGE = "syringe";
exports.TAG = "tag";
exports.TAKE_ACTION = "take-action";
exports.TANK = "tank";
exports.TARGET = "target";
exports.TAXI = "taxi";
exports.TEMPERATURE = "temperature";
exports.TEXT_HIGHLIGHT = "text-highlight";
exports.TH = "th";
exports.TH_DERIVED = "th-derived";
exports.TH_DISCONNECT = "th-disconnect";
exports.TH_FILTERED = "th-filtered";
exports.TH_LIST = "th-list";
exports.THIRD_PARTY = "third-party";
exports.THUMBS_DOWN = "thumbs-down";
exports.THUMBS_UP = "thumbs-up";
exports.TICK = "tick";
exports.TICK_CIRCLE = "tick-circle";
exports.TIME = "time";
exports.TIMELINE_AREA_CHART = "timeline-area-chart";
exports.TIMELINE_BAR_CHART = "timeline-bar-chart";
exports.TIMELINE_EVENTS = "timeline-events";
exports.TIMELINE_LINE_CHART = "timeline-line-chart";
exports.TINT = "tint";
exports.TORCH = "torch";
exports.TRACTOR = "tractor";
exports.TRAIN = "train";
exports.TRANSLATE = "translate";
exports.TRASH = "trash";
exports.TREE = "tree";
exports.TRENDING_DOWN = "trending-down";
exports.TRENDING_UP = "trending-up";
exports.TRUCK = "truck";
exports.TWO_COLUMNS = "two-columns";
exports.UNARCHIVE = "unarchive";
exports.UNDERLINE = "underline";
exports.UNDO = "undo";
exports.UNGROUP_OBJECTS = "ungroup-objects";
exports.UNKNOWN_VEHICLE = "unknown-vehicle";
exports.UNLOCK = "unlock";
exports.UNPIN = "unpin";
exports.UNRESOLVE = "unresolve";
exports.UPDATED = "updated";
exports.UPLOAD = "upload";
exports.USER = "user";
exports.VARIABLE = "variable";
exports.VERTICAL_BAR_CHART_ASC = "vertical-bar-chart-asc";
exports.VERTICAL_BAR_CHART_DESC = "vertical-bar-chart-desc";
exports.VERTICAL_DISTRIBUTION = "vertical-distribution";
exports.VIDEO = "video";
exports.VIRUS = "virus";
exports.VOLUME_DOWN = "volume-down";
exports.VOLUME_OFF = "volume-off";
exports.VOLUME_UP = "volume-up";
exports.WALK = "walk";
exports.WARNING_SIGN = "warning-sign";
exports.WATERFALL_CHART = "waterfall-chart";
exports.WAVES = "waves";
exports.WIDGET = "widget";
exports.WIDGET_BUTTON = "widget-button";
exports.WIDGET_FOOTER = "widget-footer";
exports.WIDGET_HEADER = "widget-header";
exports.WIND = "wind";
exports.WRENCH = "wrench";
exports.ZOOM_IN = "zoom-in";
exports.ZOOM_OUT = "zoom-out";
exports.ZOOM_TO_FIT = "zoom-to-fit";
//# sourceMappingURL=iconNames.js.map