/*!
 * Copyright 2016 The ANTLR Project. All rights reserved.
 * Licensed under the BSD-3-Clause license. See LICENSE file in the project root for license information.
 */
/**
 * Represents the type of recognizer an ATN applies to.
 *
 * @author Sam Harwell
 */
export declare const enum ATNType {
    /**
     * A lexer grammar.
     */
    LEXER = 0,
    /**
     * A parser grammar.
     */
    PARSER = 1
}
