import Displayable from '../graphic/Displayable';
import { BrushScope, SVGVNodeAttrs } from './core';
export declare function createCSSEmphasis(el: Displayable, attrs: SVGVNodeAttrs, scope: BrushScope): void;
