export { default as oneC } from './1c';
export { default as abnf } from './abnf';
export { default as accesslog } from './accesslog';
export { default as actionscript } from './actionscript';
export { default as ada } from './ada';
export { default as angelscript } from './angelscript';
export { default as apache } from './apache';
export { default as applescript } from './applescript';
export { default as arcade } from './arcade';
export { default as arduino } from './arduino';
export { default as armasm } from './armasm';
export { default as asciidoc } from './asciidoc';
export { default as aspectj } from './aspectj';
export { default as autohotkey } from './autohotkey';
export { default as autoit } from './autoit';
export { default as avrasm } from './avrasm';
export { default as awk } from './awk';
export { default as axapta } from './axapta';
export { default as bash } from './bash';
export { default as basic } from './basic';
export { default as bnf } from './bnf';
export { default as brainfuck } from './brainfuck';
export { default as cLike } from './c-like';
export { default as c } from './c';
export { default as cal } from './cal';
export { default as capnproto } from './capnproto';
export { default as ceylon } from './ceylon';
export { default as clean } from './clean';
export { default as clojureRepl } from './clojure-repl';
export { default as clojure } from './clojure';
export { default as cmake } from './cmake';
export { default as coffeescript } from './coffeescript';
export { default as coq } from './coq';
export { default as cos } from './cos';
export { default as cpp } from './cpp';
export { default as crmsh } from './crmsh';
export { default as crystal } from './crystal';
export { default as csharp } from './csharp';
export { default as csp } from './csp';
export { default as css } from './css';
export { default as d } from './d';
export { default as dart } from './dart';
export { default as delphi } from './delphi';
export { default as diff } from './diff';
export { default as django } from './django';
export { default as dns } from './dns';
export { default as dockerfile } from './dockerfile';
export { default as dos } from './dos';
export { default as dsconfig } from './dsconfig';
export { default as dts } from './dts';
export { default as dust } from './dust';
export { default as ebnf } from './ebnf';
export { default as elixir } from './elixir';
export { default as elm } from './elm';
export { default as erb } from './erb';
export { default as erlangRepl } from './erlang-repl';
export { default as erlang } from './erlang';
export { default as excel } from './excel';
export { default as fix } from './fix';
export { default as flix } from './flix';
export { default as fortran } from './fortran';
export { default as fsharp } from './fsharp';
export { default as gams } from './gams';
export { default as gauss } from './gauss';
export { default as gcode } from './gcode';
export { default as gherkin } from './gherkin';
export { default as glsl } from './glsl';
export { default as gml } from './gml';
export { default as go } from './go';
export { default as golo } from './golo';
export { default as gradle } from './gradle';
export { default as groovy } from './groovy';
export { default as haml } from './haml';
export { default as handlebars } from './handlebars';
export { default as haskell } from './haskell';
export { default as haxe } from './haxe';
export { default as hsp } from './hsp';
export { default as htmlbars } from './htmlbars';
export { default as http } from './http';
export { default as hy } from './hy';
export { default as inform7 } from './inform7';
export { default as ini } from './ini';
export { default as irpf90 } from './irpf90';
export { default as isbl } from './isbl';
export { default as java } from './java';
export { default as javascript } from './javascript';
export { default as jbossCli } from './jboss-cli';
export { default as json } from './json';
export { default as juliaRepl } from './julia-repl';
export { default as julia } from './julia';
export { default as kotlin } from './kotlin';
export { default as lasso } from './lasso';
export { default as latex } from './latex';
export { default as ldif } from './ldif';
export { default as leaf } from './leaf';
export { default as less } from './less';
export { default as lisp } from './lisp';
export { default as livecodeserver } from './livecodeserver';
export { default as livescript } from './livescript';
export { default as llvm } from './llvm';
export { default as lsl } from './lsl';
export { default as lua } from './lua';
export { default as makefile } from './makefile';
export { default as markdown } from './markdown';
export { default as mathematica } from './mathematica';
export { default as matlab } from './matlab';
export { default as maxima } from './maxima';
export { default as mel } from './mel';
export { default as mercury } from './mercury';
export { default as mipsasm } from './mipsasm';
export { default as mizar } from './mizar';
export { default as mojolicious } from './mojolicious';
export { default as monkey } from './monkey';
export { default as moonscript } from './moonscript';
export { default as n1ql } from './n1ql';
export { default as nginx } from './nginx';
export { default as nim } from './nim';
export { default as nix } from './nix';
export { default as nodeRepl } from './node-repl';
export { default as nsis } from './nsis';
export { default as objectivec } from './objectivec';
export { default as ocaml } from './ocaml';
export { default as openscad } from './openscad';
export { default as oxygene } from './oxygene';
export { default as parser3 } from './parser3';
export { default as perl } from './perl';
export { default as pf } from './pf';
export { default as pgsql } from './pgsql';
export { default as phpTemplate } from './php-template';
export { default as php } from './php';
export { default as plaintext } from './plaintext';
export { default as pony } from './pony';
export { default as powershell } from './powershell';
export { default as processing } from './processing';
export { default as profile } from './profile';
export { default as prolog } from './prolog';
export { default as properties } from './properties';
export { default as protobuf } from './protobuf';
export { default as puppet } from './puppet';
export { default as purebasic } from './purebasic';
export { default as pythonRepl } from './python-repl';
export { default as python } from './python';
export { default as q } from './q';
export { default as qml } from './qml';
export { default as r } from './r';
export { default as reasonml } from './reasonml';
export { default as rib } from './rib';
export { default as roboconf } from './roboconf';
export { default as routeros } from './routeros';
export { default as rsl } from './rsl';
export { default as ruby } from './ruby';
export { default as ruleslanguage } from './ruleslanguage';
export { default as rust } from './rust';
export { default as sas } from './sas';
export { default as scala } from './scala';
export { default as scheme } from './scheme';
export { default as scilab } from './scilab';
export { default as scss } from './scss';
export { default as shell } from './shell';
export { default as smali } from './smali';
export { default as smalltalk } from './smalltalk';
export { default as sml } from './sml';
export { default as sqf } from './sqf';
export { default as sql } from './sql';
export { default as sqlMore } from './sql_more';
export { default as stan } from './stan';
export { default as stata } from './stata';
export { default as step21 } from './step21';
export { default as stylus } from './stylus';
export { default as subunit } from './subunit';
export { default as swift } from './swift';
export { default as taggerscript } from './taggerscript';
export { default as tap } from './tap';
export { default as tcl } from './tcl';
export { default as thrift } from './thrift';
export { default as tp } from './tp';
export { default as twig } from './twig';
export { default as typescript } from './typescript';
export { default as vala } from './vala';
export { default as vbnet } from './vbnet';
export { default as vbscriptHtml } from './vbscript-html';
export { default as vbscript } from './vbscript';
export { default as verilog } from './verilog';
export { default as vhdl } from './vhdl';
export { default as vim } from './vim';
export { default as x86asm } from './x86asm';
export { default as xl } from './xl';
export { default as xml } from './xml';
export { default as xquery } from './xquery';
export { default as yaml } from './yaml';
export { default as zephir } from './zephir';