import View from '../../coord/View.js';
import ExtensionAPI from '../../core/ExtensionAPI.js';
import GlobalModel from '../../model/Global.js';
export default function createViewCoordSys(ecModel: GlobalModel, api: ExtensionAPI): View[];
