"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = __importStar(require("react"));
const rich_media_1 = require("./rich-media");
exports.ExecuteResult = (props) => {
    const { output, children } = props;
    if (!output) {
        return null;
    }
    return (React.createElement(rich_media_1.RichMedia, { data: output.data, metadata: output.metadata }, children));
};
exports.ExecuteResult.defaultProps = {
    output: null,
    output_type: "execute_result"
};
exports.default = exports.ExecuteResult;
