/**
 * Escape carrigage returns like a terminal
 */
export function escapeCarriageReturn(text: string): string;
export default escapeCarriageReturn;

/**
 * Safely escape carrigage returns like a terminal.
 * This allows to escape carrigage returns while allowing future output to be appended
 * without loosing information.
 * Use this as a intermediate escape step if your stream hasn't completed yet.
 */
export function escapeCarriageReturnSafe(text: string): string;
