declare const isSVG: (target: Element) => boolean;
declare const isHidden: (target: Element) => boolean;
declare const isElement: (obj: unknown) => boolean;
declare const isReplacedElement: (target: Element) => boolean;
export { isSVG, isHidden, isElement, isReplacedElement };
