interface ErrorProps {
    className?: string;
    error?: Error;
}
export declare const Error: (props: ErrorProps) => JSX.Element | null;
declare const StyledError: import("styled-components").StyledComponent<(props: ErrorProps) => JSX.Element | null, any, {}, never>;
export default StyledError;
