"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const markdown_1 = __importDefault(require("@nteract/markdown"));
const React = __importStar(require("react"));
class Markdown extends React.PureComponent {
    render() {
        return React.createElement(markdown_1.default, { source: this.props.data });
    }
}
exports.Markdown = Markdown;
Markdown.defaultProps = {
    data: "",
    mediaType: "text/markdown"
};
exports.default = Markdown;
