/*
 * Copyright 2021 Palantir Technologies, Inc. All rights reserved.
 */

$black: #10161a !default;

$dark-gray1: #182026 !default;
$dark-gray2: #202b33 !default;
$dark-gray3: #293742 !default;
$dark-gray4: #30404d !default;
$dark-gray5: #394b59 !default;

$gray1: #5c7080 !default;
$gray2: #738694 !default;
$gray3: #8a9ba8 !default;
$gray4: #a7b6c2 !default;
$gray5: #bfccd6 !default;

$light-gray1: #ced9e0 !default;
$light-gray2: #d8e1e8 !default;
$light-gray3: #e1e8ed !default;
$light-gray4: #ebf1f5 !default;
$light-gray5: #f5f8fa !default;

$white: #ffffff !default;

$blue1: #0e5a8a !default;
$blue2: #106ba3 !default;
$blue3: #137cbd !default;
$blue4: #2b95d6 !default;
$blue5: #48aff0 !default;

$green1: #0a6640 !default;
$green2: #0d8050 !default;
$green3: #0f9960 !default;
$green4: #15b371 !default;
$green5: #3dcc91 !default;

$orange1: #a66321 !default;
$orange2: #bf7326 !default;
$orange3: #d9822b !default;
$orange4: #f29d49 !default;
$orange5: #ffb366 !default;

$red1: #a82a2a !default;
$red2: #c23030 !default;
$red3: #db3737 !default;
$red4: #f55656 !default;
$red5: #ff7373 !default;

$vermilion1: #9e2b0e !default;
$vermilion2: #b83211 !default;
$vermilion3: #d13913 !default;
$vermilion4: #eb532d !default;
$vermilion5: #ff6e4a !default;

$rose1: #a82255 !default;
$rose2: #c22762 !default;
$rose3: #db2c6f !default;
$rose4: #f5498b !default;
$rose5: #ff66a1 !default;

$violet1: #5c255c !default;
$violet2: #752f75 !default;
$violet3: #8f398f !default;
$violet4: #a854a8 !default;
$violet5: #c274c2 !default;

$indigo1: #5642a6 !default;
$indigo2: #634dbf !default;
$indigo3: #7157d9 !default;
$indigo4: #9179f2 !default;
$indigo5: #ad99ff !default;

$cobalt1: #1f4b99 !default;
$cobalt2: #2458b3 !default;
$cobalt3: #2965cc !default;
$cobalt4: #4580e6 !default;
$cobalt5: #669eff !default;

$turquoise1: #008075 !default;
$turquoise2: #00998c !default;
$turquoise3: #00b3a4 !default;
$turquoise4: #14ccbd !default;
$turquoise5: #2ee6d6 !default;

$forest1: #1d7324 !default;
$forest2: #238c2c !default;
$forest3: #29a634 !default;
$forest4: #43bf4d !default;
$forest5: #62d96b !default;

$lime1: #728c23 !default;
$lime2: #87a629 !default;
$lime3: #9bbf30 !default;
$lime4: #b6d94c !default;
$lime5: #d1f26d !default;

$gold1: #a67908 !default;
$gold2: #bf8c0a !default;
$gold3: #d99e0b !default;
$gold4: #f2b824 !default;
$gold5: #ffc940 !default;

$sepia1: #63411e !default;
$sepia2: #7d5125 !default;
$sepia3: #96622d !default;
$sepia4: #b07b46 !default;
$sepia5: #c99765 !default;

$pt-intent-primary: $blue3 !default;
$pt-intent-success: $green3 !default;
$pt-intent-warning: $orange3 !default;
$pt-intent-danger: $red3 !default;

$pt-app-background-color: $light-gray5 !default;
$pt-dark-app-background-color: $dark-gray3 !default;

$pt-outline-color: rgba($blue3, 0.6);

$pt-text-color: $dark-gray1 !default;
$pt-text-color-muted: $gray1 !default;
$pt-text-color-disabled: rgba($pt-text-color-muted, 0.6) !default;
$pt-heading-color: $pt-text-color !default;
$pt-link-color: $blue2 !default;
$pt-dark-text-color: $light-gray5 !default;
$pt-dark-text-color-muted: $gray4 !default;
$pt-dark-text-color-disabled: rgba($pt-dark-text-color-muted, 0.6) !default;
$pt-dark-heading-color: $pt-dark-text-color !default;
$pt-dark-link-color: $blue5 !default;
$pt-text-selection-color: rgba(125, 188, 255, 0.6) !default;

$pt-icon-color: $pt-text-color-muted !default;
$pt-icon-color-hover: $pt-text-color !default;
$pt-icon-color-disabled: $pt-text-color-disabled !default;
$pt-icon-color-selected: $pt-intent-primary !default;
$pt-dark-icon-color: $pt-dark-text-color-muted !default;
$pt-dark-icon-color-hover: $pt-dark-text-color !default;
$pt-dark-icon-color-disabled: $pt-dark-text-color-disabled !default;
$pt-dark-icon-color-selected: $pt-intent-primary !default;

$pt-divider-black: rgba($black, 0.15) !default;
$pt-dark-divider-black: rgba($black, 0.4) !default;
$pt-dark-divider-white: rgba($white, 0.15) !default;

$pt-code-text-color: $pt-text-color-muted !default;
$pt-dark-code-text-color: $pt-dark-text-color-muted !default;
$pt-code-background-color: rgba($white, 0.7) !default;
$pt-dark-code-background-color: rgba($black, 0.3) !default;

$cerulean1: $cobalt1;
$cerulean2: $cobalt2;
$cerulean3: $cobalt3;
$cerulean4: $cobalt4;
$cerulean5: $cobalt5;
$ns: bp3 !default;
$bp-ns: $ns;

$pt-grid-size: 10px !default;

$pt-font-family: -apple-system,
                 "BlinkMacSystemFont",
                 "Segoe UI",
                 "Roboto",
                 "Oxygen",
                 "Ubuntu",
                 "Cantarell",
                 "Open Sans",
                 "Helvetica Neue",
                 "Icons16",                  sans-serif !default;

$pt-font-family-monospace: monospace !default;

$pt-font-size: $pt-grid-size * 1.4 !default;
$pt-font-size-large: $pt-grid-size * 1.6 !default;
$pt-font-size-small: $pt-grid-size * 1.2 !default;

$pt-line-height: ($pt-grid-size * 1.8) / $pt-font-size + 0.0001 !default;

$icons16-family: "Icons16" !default;
$icons20-family: "Icons20" !default;

$pt-icon-size-standard: 16px !default;
$pt-icon-size-large: 20px !default;

$pt-border-radius: floor($pt-grid-size / 3) !default;

$pt-button-height: $pt-grid-size * 3 !default;
$pt-button-height-small: $pt-grid-size * 2.4 !default;
$pt-button-height-smaller: $pt-grid-size * 2 !default;
$pt-button-height-large: $pt-grid-size * 4 !default;

$pt-input-height: $pt-grid-size * 3 !default;
$pt-input-height-large: $pt-grid-size * 4 !default;
$pt-input-height-small: $pt-grid-size * 2.4 !default;

$pt-navbar-height: $pt-grid-size * 5 !default;

$pt-z-index-base: 0 !default;
$pt-z-index-content: $pt-z-index-base + 10 !default;
$pt-z-index-overlay: $pt-z-index-content + 10 !default;
$pt-z-index-dialog-header: $pt-z-index-overlay + 10 !default;

$pt-border-shadow-opacity: 0.1 !default;
$pt-drop-shadow-opacity: 0.2 !default;
$pt-dark-border-shadow-opacity: $pt-border-shadow-opacity * 2 !default;
$pt-dark-drop-shadow-opacity: $pt-drop-shadow-opacity * 2 !default;

$pt-elevation-shadow-0: 0 0 0 1px $pt-divider-black,
                        0 0 0 rgba($black, 0),
                        0 0 0 rgba($black, 0) !default;
$pt-elevation-shadow-1: 0 0 0 1px rgba($black, $pt-border-shadow-opacity),
                        0 0 0 rgba($black, 0),
                        0 1px 1px rgba($black, $pt-drop-shadow-opacity) !default;
$pt-elevation-shadow-2: 0 0 0 1px rgba($black, $pt-border-shadow-opacity),
                        0 1px 1px rgba($black, $pt-drop-shadow-opacity),
                        0 2px 6px rgba($black, $pt-drop-shadow-opacity) !default;
$pt-elevation-shadow-3: 0 0 0 1px rgba($black, $pt-border-shadow-opacity),
                        0 2px 4px rgba($black, $pt-drop-shadow-opacity),
                        0 8px 24px rgba($black, $pt-drop-shadow-opacity) !default;
$pt-elevation-shadow-4: 0 0 0 1px rgba($black, $pt-border-shadow-opacity),
                        0 4px 8px rgba($black, $pt-drop-shadow-opacity),
                        0 18px 46px 6px rgba($black, $pt-drop-shadow-opacity) !default;

$pt-dark-elevation-shadow-0: 0 0 0 1px $pt-dark-divider-black,
                             0 0 0 rgba($black, 0),
                             0 0 0 rgba($black, 0) !default;
$pt-dark-elevation-shadow-1: 0 0 0 1px rgba($black, $pt-dark-border-shadow-opacity),
                             0 0 0 rgba($black, 0),
                             0 1px 1px rgba($black, $pt-dark-drop-shadow-opacity) !default;
$pt-dark-elevation-shadow-2: 0 0 0 1px rgba($black, $pt-dark-border-shadow-opacity),
                             0 1px 1px rgba($black, $pt-dark-drop-shadow-opacity),
                             0 2px 6px rgba($black, $pt-dark-drop-shadow-opacity) !default;
$pt-dark-elevation-shadow-3: 0 0 0 1px rgba($black, $pt-dark-border-shadow-opacity),
                             0 2px 4px rgba($black, $pt-dark-drop-shadow-opacity),
                             0 8px 24px rgba($black, $pt-dark-drop-shadow-opacity) !default;
$pt-dark-elevation-shadow-4: 0 0 0 1px rgba($black, $pt-dark-border-shadow-opacity),
                             0 4px 8px rgba($black, $pt-dark-drop-shadow-opacity),
                             0 18px 46px 6px rgba($black, $pt-dark-drop-shadow-opacity) !default;

$pt-transition-ease: cubic-bezier(0.4, 1, 0.75, 0.9) !default;
$pt-transition-ease-bounce: cubic-bezier(0.54, 1.12, 0.38, 1.11) !default;
$pt-transition-duration: 100ms !default;

$pt-input-box-shadow: inset 0 0 0 1px rgba($black, 0.15),
                      inset 0 1px 1px rgba($black, $pt-drop-shadow-opacity) !default;

$pt-dialog-box-shadow: $pt-elevation-shadow-4 !default;
$pt-popover-box-shadow: $pt-elevation-shadow-3 !default;
$pt-tooltip-box-shadow: $pt-popover-box-shadow !default;

$pt-dark-input-box-shadow: inset 0 0 0 1px rgba($black, 0.3),
                           inset 0 1px 1px rgba($black, $pt-dark-drop-shadow-opacity) !default;

$pt-dark-dialog-box-shadow: $pt-dark-elevation-shadow-4 !default;
$pt-dark-popover-box-shadow: $pt-dark-elevation-shadow-3 !default;
$pt-dark-tooltip-box-shadow: $pt-dark-popover-box-shadow !default;
