export declare const warnCache: {
    current: {};
};
/**
 * Logs a warning if the condition is not met.
 * This is used to log issues in development environment only.
 */
export declare function warn(condition: boolean, message: string): void;
