export declare const ADD = "\uE63E";
export declare const ADD_COLUMN_LEFT = "\uE6F9";
export declare const ADD_COLUMN_RIGHT = "\uE6FA";
export declare const ADD_ROW_BOTTOM = "\uE6F8";
export declare const ADD_ROW_TOP = "\uE6F7";
export declare const ADD_TO_ARTIFACT = "\uE67C";
export declare const ADD_TO_FOLDER = "\uE6D2";
export declare const AIRPLANE = "\uE74B";
export declare const ALIGN_CENTER = "\uE603";
export declare const ALIGN_JUSTIFY = "\uE605";
export declare const ALIGN_LEFT = "\uE602";
export declare const ALIGN_RIGHT = "\uE604";
export declare const ALIGNMENT_BOTTOM = "\uE727";
export declare const ALIGNMENT_HORIZONTAL_CENTER = "\uE726";
export declare const ALIGNMENT_LEFT = "\uE722";
export declare const ALIGNMENT_RIGHT = "\uE724";
export declare const ALIGNMENT_TOP = "\uE725";
export declare const ALIGNMENT_VERTICAL_CENTER = "\uE723";
export declare const ANNOTATION = "\uE6F0";
export declare const APPLICATION = "\uE735";
export declare const APPLICATIONS = "\uE621";
export declare const ARCHIVE = "\uE907";
export declare const ARROW_BOTTOM_LEFT = "\u2199";
export declare const ARROW_BOTTOM_RIGHT = "\u2198";
export declare const ARROW_DOWN = "\u2193";
export declare const ARROW_LEFT = "\u2190";
export declare const ARROW_RIGHT = "\u2192";
export declare const ARROW_TOP_LEFT = "\u2196";
export declare const ARROW_TOP_RIGHT = "\u2197";
export declare const ARROW_UP = "\u2191";
export declare const ARROWS_HORIZONTAL = "\u2194";
export declare const ARROWS_VERTICAL = "\u2195";
export declare const ASTERISK = "*";
export declare const AUTOMATIC_UPDATES = "\uE65F";
export declare const BADGE = "\uE6E3";
export declare const BAN_CIRCLE = "\uE69D";
export declare const BANK_ACCOUNT = "\uE76F";
export declare const BARCODE = "\uE676";
export declare const BLANK = "\uE900";
export declare const BLOCKED_PERSON = "\uE768";
export declare const BOLD = "\uE606";
export declare const BOOK = "\uE6B8";
export declare const BOOKMARK = "\uE61A";
export declare const BOX = "\uE6BF";
export declare const BRIEFCASE = "\uE674";
export declare const BRING_DATA = "\uE90A";
export declare const BUILD = "\uE72D";
export declare const CALCULATOR = "\uE70B";
export declare const CALENDAR = "\uE62B";
export declare const CAMERA = "\uE69E";
export declare const CARET_DOWN = "\u2304";
export declare const CARET_LEFT = "\u2329";
export declare const CARET_RIGHT = "\u232A";
export declare const CARET_UP = "\u2303";
export declare const CELL_TOWER = "\uE770";
export declare const CHANGES = "\uE623";
export declare const CHART = "\uE67E";
export declare const CHAT = "\uE689";
export declare const CHEVRON_BACKWARD = "\uE6DF";
export declare const CHEVRON_DOWN = "\uE697";
export declare const CHEVRON_FORWARD = "\uE6E0";
export declare const CHEVRON_LEFT = "\uE694";
export declare const CHEVRON_RIGHT = "\uE695";
export declare const CHEVRON_UP = "\uE696";
export declare const CIRCLE = "\uE66A";
export declare const CIRCLE_ARROW_DOWN = "\uE68E";
export declare const CIRCLE_ARROW_LEFT = "\uE68C";
export declare const CIRCLE_ARROW_RIGHT = "\uE68B";
export declare const CIRCLE_ARROW_UP = "\uE68D";
export declare const CITATION = "\uE61B";
export declare const CLEAN = "\uE7C5";
export declare const CLIPBOARD = "\uE61D";
export declare const CLOUD = "\u2601";
export declare const CLOUD_DOWNLOAD = "\uE690";
export declare const CLOUD_UPLOAD = "\uE691";
export declare const CODE = "\uE661";
export declare const CODE_BLOCK = "\uE6C5";
export declare const COG = "\uE645";
export declare const COLLAPSE_ALL = "\uE763";
export declare const COLUMN_LAYOUT = "\uE6DA";
export declare const COMMENT = "\uE68A";
export declare const COMPARISON = "\uE637";
export declare const COMPASS = "\uE79C";
export declare const COMPRESSED = "\uE6C0";
export declare const CONFIRM = "\uE639";
export declare const CONSOLE = "\uE79B";
export declare const CONTRAST = "\uE6CB";
export declare const CONTROL = "\uE67F";
export declare const CREDIT_CARD = "\uE649";
export declare const CROSS = "\u2717";
export declare const CROWN = "\uE7B4";
export declare const CUBE = "\uE7C8";
export declare const CUBE_ADD = "\uE7C9";
export declare const CUBE_REMOVE = "\uE7D0";
export declare const CURVED_RANGE_CHART = "\uE71B";
export declare const CUT = "\uE6EF";
export declare const DASHBOARD = "\uE751";
export declare const DATA_LINEAGE = "\uE908";
export declare const DATABASE = "\uE683";
export declare const DELETE = "\uE644";
export declare const DELTA = "\u0394";
export declare const DERIVE_COLUMN = "\uE739";
export declare const DESKTOP = "\uE6AF";
export declare const DIAGNOSIS = "\uE90D";
export declare const DIAGRAM_TREE = "\uE7B3";
export declare const DIRECTION_LEFT = "\uE681";
export declare const DIRECTION_RIGHT = "\uE682";
export declare const DISABLE = "\uE600";
export declare const DOCUMENT = "\uE630";
export declare const DOCUMENT_OPEN = "\uE71E";
export declare const DOCUMENT_SHARE = "\uE71F";
export declare const DOLLAR = "$";
export declare const DOT = "\u2022";
export declare const DOUBLE_CARET_HORIZONTAL = "\uE6C7";
export declare const DOUBLE_CARET_VERTICAL = "\uE6C6";
export declare const DOUBLE_CHEVRON_DOWN = "\uE703";
export declare const DOUBLE_CHEVRON_LEFT = "\uE6FF";
export declare const DOUBLE_CHEVRON_RIGHT = "\uE701";
export declare const DOUBLE_CHEVRON_UP = "\uE702";
export declare const DOUGHNUT_CHART = "\uE6CE";
export declare const DOWNLOAD = "\uE62F";
export declare const DRAG_HANDLE_HORIZONTAL = "\uE716";
export declare const DRAG_HANDLE_VERTICAL = "\uE715";
export declare const DRAW = "\uE66B";
export declare const DRIVE_TIME = "\uE615";
export declare const DUPLICATE = "\uE69C";
export declare const EDIT = "\u270E";
export declare const EJECT = "\u23CF";
export declare const ENDORSED = "\uE75F";
export declare const ENVELOPE = "\u2709";
export declare const EQUALS = "\uE7D9";
export declare const ERASER = "\uE773";
export declare const ERROR = "\uE648";
export declare const EURO = "\u20AC";
export declare const EXCHANGE = "\uE636";
export declare const EXCLUDE_ROW = "\uE6EA";
export declare const EXPAND_ALL = "\uE764";
export declare const EXPORT = "\uE633";
export declare const EYE_OFF = "\uE6CC";
export declare const EYE_ON = "\uE75A";
export declare const EYE_OPEN = "\uE66F";
export declare const FAST_BACKWARD = "\uE6A8";
export declare const FAST_FORWARD = "\uE6AC";
export declare const FEED = "\uE656";
export declare const FEED_SUBSCRIBED = "\uE78F";
export declare const FILM = "\uE6A1";
export declare const FILTER = "\uE638";
export declare const FILTER_KEEP = "\uE78C";
export declare const FILTER_LIST = "\uE6EE";
export declare const FILTER_OPEN = "\uE7D7";
export declare const FILTER_REMOVE = "\uE78D";
export declare const FLAG = "\u2691";
export declare const FLAME = "\uE7A9";
export declare const FLASH = "\uE6B3";
export declare const FLOPPY_DISK = "\uE6B7";
export declare const FLOW_BRANCH = "\uE7C1";
export declare const FLOW_END = "\uE7C4";
export declare const FLOW_LINEAR = "\uE7C0";
export declare const FLOW_REVIEW = "\uE7C2";
export declare const FLOW_REVIEW_BRANCH = "\uE7C3";
export declare const FLOWS = "\uE659";
export declare const FOLDER_CLOSE = "\uE652";
export declare const FOLDER_NEW = "\uE7B0";
export declare const FOLDER_OPEN = "\uE651";
export declare const FOLDER_SHARED = "\uE653";
export declare const FOLDER_SHARED_OPEN = "\uE670";
export declare const FOLLOWER = "\uE760";
export declare const FOLLOWING = "\uE761";
export declare const FONT = "\uE6B4";
export declare const FORK = "\uE63A";
export declare const FORM = "\uE795";
export declare const FULL_CIRCLE = "\uE685";
export declare const FULL_STACKED_CHART = "\uE75E";
export declare const FULLSCREEN = "\uE699";
export declare const FUNCTION = "\uE6E5";
export declare const GANTT_CHART = "\uE6F4";
export declare const GEOLOCATION = "\uE640";
export declare const GEOSEARCH = "\uE613";
export declare const GIT_BRANCH = "\uE72A";
export declare const GIT_COMMIT = "\uE72B";
export declare const GIT_MERGE = "\uE729";
export declare const GIT_NEW_BRANCH = "\uE749";
export declare const GIT_PULL = "\uE728";
export declare const GIT_PUSH = "\uE72C";
export declare const GIT_REPO = "\uE748";
export declare const GLASS = "\uE6B1";
export declare const GLOBE = "\uE666";
export declare const GLOBE_NETWORK = "\uE7B5";
export declare const GRAPH = "\uE673";
export declare const GRAPH_REMOVE = "\uE609";
export declare const GREATER_THAN = "\uE7E1";
export declare const GREATER_THAN_OR_EQUAL_TO = "\uE7E2";
export declare const GRID = "\uE6D0";
export declare const GRID_VIEW = "\uE6E4";
export declare const GROUP_OBJECTS = "\uE60A";
export declare const GROUPED_BAR_CHART = "\uE75D";
export declare const HAND = "\uE6DE";
export declare const HAND_DOWN = "\uE6BB";
export declare const HAND_LEFT = "\uE6BC";
export declare const HAND_RIGHT = "\uE6B9";
export declare const HAND_UP = "\uE6BA";
export declare const HEADER = "\uE6B5";
export declare const HEADER_ONE = "\uE793";
export declare const HEADER_TWO = "\uE794";
export declare const HEADSET = "\uE6DC";
export declare const HEART = "\u2665";
export declare const HEART_BROKEN = "\uE7A2";
export declare const HEAT_GRID = "\uE6F3";
export declare const HEATMAP = "\uE614";
export declare const HELP = "?";
export declare const HELPER_MANAGEMENT = "\uE66D";
export declare const HIGHLIGHT = "\uE6ED";
export declare const HISTORY = "\uE64A";
export declare const HOME = "\u2302";
export declare const HORIZONTAL_BAR_CHART = "\uE70C";
export declare const HORIZONTAL_BAR_CHART_ASC = "\uE75C";
export declare const HORIZONTAL_BAR_CHART_DESC = "\uE71D";
export declare const HORIZONTAL_DISTRIBUTION = "\uE720";
export declare const ID_NUMBER = "\uE771";
export declare const IMAGE_ROTATE_LEFT = "\uE73A";
export declare const IMAGE_ROTATE_RIGHT = "\uE73B";
export declare const IMPORT = "\uE632";
export declare const INBOX = "\uE629";
export declare const INBOX_FILTERED = "\uE7D1";
export declare const INBOX_GEO = "\uE7D2";
export declare const INBOX_SEARCH = "\uE7D3";
export declare const INBOX_UPDATE = "\uE7D4";
export declare const INFO_SIGN = "\u2139";
export declare const INHERITANCE = "\uE7D5";
export declare const INNER_JOIN = "\uE7A3";
export declare const INSERT = "\uE66C";
export declare const INTERSECTION = "\uE765";
export declare const IP_ADDRESS = "\uE772";
export declare const ISSUE = "\uE774";
export declare const ISSUE_CLOSED = "\uE776";
export declare const ISSUE_NEW = "\uE775";
export declare const ITALIC = "\uE607";
export declare const JOIN_TABLE = "\uE738";
export declare const KEY = "\uE78E";
export declare const KEY_BACKSPACE = "\uE707";
export declare const KEY_COMMAND = "\uE705";
export declare const KEY_CONTROL = "\uE704";
export declare const KEY_DELETE = "\uE708";
export declare const KEY_ENTER = "\uE70A";
export declare const KEY_ESCAPE = "\uE709";
export declare const KEY_OPTION = "\uE742";
export declare const KEY_SHIFT = "\uE706";
export declare const KEY_TAB = "\uE757";
export declare const KNOWN_VEHICLE = "\uE73C";
export declare const LAB_TEST = "\uE90E";
export declare const LABEL = "\uE665";
export declare const LAYER = "\uE6CF";
export declare const LAYERS = "\uE618";
export declare const LAYOUT = "\uE60C";
export declare const LAYOUT_AUTO = "\uE60D";
export declare const LAYOUT_BALLOON = "\uE6D3";
export declare const LAYOUT_CIRCLE = "\uE60E";
export declare const LAYOUT_GRID = "\uE610";
export declare const LAYOUT_GROUP_BY = "\uE611";
export declare const LAYOUT_HIERARCHY = "\uE60F";
export declare const LAYOUT_LINEAR = "\uE6C3";
export declare const LAYOUT_SKEW_GRID = "\uE612";
export declare const LAYOUT_SORTED_CLUSTERS = "\uE6D4";
export declare const LEARNING = "\uE904";
export declare const LEFT_JOIN = "\uE7A4";
export declare const LESS_THAN = "\uE7E3";
export declare const LESS_THAN_OR_EQUAL_TO = "\uE7E4";
export declare const LIFESAVER = "\uE7C7";
export declare const LIGHTBULB = "\uE6B0";
export declare const LINK = "\uE62D";
export declare const LIST = "\u2630";
export declare const LIST_COLUMNS = "\uE7B9";
export declare const LIST_DETAIL_VIEW = "\uE743";
export declare const LOCATE = "\uE619";
export declare const LOCK = "\uE625";
export declare const LOG_IN = "\uE69A";
export declare const LOG_OUT = "\uE64C";
export declare const MANUAL = "\uE6F6";
export declare const MANUALLY_ENTERED_DATA = "\uE74A";
export declare const MAP = "\uE662";
export declare const MAP_CREATE = "\uE741";
export declare const MAP_MARKER = "\uE67D";
export declare const MAXIMIZE = "\uE635";
export declare const MEDIA = "\uE62C";
export declare const MENU = "\uE762";
export declare const MENU_CLOSED = "\uE655";
export declare const MENU_OPEN = "\uE654";
export declare const MERGE_COLUMNS = "\uE74F";
export declare const MERGE_LINKS = "\uE60B";
export declare const MINIMIZE = "\uE634";
export declare const MINUS = "\u2212";
export declare const MOBILE_PHONE = "\uE717";
export declare const MOBILE_VIDEO = "\uE69F";
export declare const MOON = "\uE754";
export declare const MORE = "\uE62A";
export declare const MOUNTAIN = "\uE7B1";
export declare const MOVE = "\uE693";
export declare const MUGSHOT = "\uE6DB";
export declare const MULTI_SELECT = "\uE680";
export declare const MUSIC = "\uE6A6";
export declare const NEW_DRAWING = "\uE905";
export declare const NEW_GRID_ITEM = "\uE747";
export declare const NEW_LAYER = "\uE902";
export declare const NEW_LAYERS = "\uE903";
export declare const NEW_LINK = "\uE65C";
export declare const NEW_OBJECT = "\uE65D";
export declare const NEW_PERSON = "\uE6E9";
export declare const NEW_PRESCRIPTION = "\uE78B";
export declare const NEW_TEXT_BOX = "\uE65B";
export declare const NINJA = "\uE675";
export declare const NOT_EQUAL_TO = "\uE7E0";
export declare const NOTIFICATIONS = "\uE624";
export declare const NOTIFICATIONS_UPDATED = "\uE7B8";
export declare const NUMBERED_LIST = "\uE746";
export declare const NUMERICAL = "\uE756";
export declare const OFFICE = "\uE69B";
export declare const OFFLINE = "\uE67A";
export declare const OIL_FIELD = "\uE73F";
export declare const ONE_COLUMN = "\uE658";
export declare const OUTDATED = "\uE7A8";
export declare const PAGE_LAYOUT = "\uE660";
export declare const PANEL_STATS = "\uE777";
export declare const PANEL_TABLE = "\uE778";
export declare const PAPERCLIP = "\uE664";
export declare const PARAGRAPH = "\uE76C";
export declare const PATH = "\uE753";
export declare const PATH_SEARCH = "\uE65E";
export declare const PAUSE = "\uE6A9";
export declare const PEOPLE = "\uE63D";
export declare const PERCENTAGE = "\uE76A";
export declare const PERSON = "\uE63C";
export declare const PHONE = "\u260E";
export declare const PIE_CHART = "\uE684";
export declare const PIN = "\uE646";
export declare const PIVOT = "\uE6F1";
export declare const PIVOT_TABLE = "\uE6EB";
export declare const PLAY = "\uE6AB";
export declare const PLUS = "+";
export declare const POLYGON_FILTER = "\uE6D1";
export declare const POWER = "\uE6D9";
export declare const PREDICTIVE_ANALYSIS = "\uE617";
export declare const PRESCRIPTION = "\uE78A";
export declare const PRESENTATION = "\uE687";
export declare const PRINT = "\u2399";
export declare const PROJECTS = "\uE622";
export declare const PROPERTIES = "\uE631";
export declare const PROPERTY = "\uE65A";
export declare const PUBLISH_FUNCTION = "\uE752";
export declare const PULSE = "\uE6E8";
export declare const RANDOM = "\uE698";
export declare const RECORD = "\uE6AE";
export declare const REDO = "\uE6C4";
export declare const REFRESH = "\uE643";
export declare const REGRESSION_CHART = "\uE758";
export declare const REMOVE = "\uE63F";
export declare const REMOVE_COLUMN = "\uE755";
export declare const REMOVE_COLUMN_LEFT = "\uE6FD";
export declare const REMOVE_COLUMN_RIGHT = "\uE6FE";
export declare const REMOVE_ROW_BOTTOM = "\uE6FC";
export declare const REMOVE_ROW_TOP = "\uE6FB";
export declare const REPEAT = "\uE692";
export declare const RESET = "\uE7D6";
export declare const RESOLVE = "\uE672";
export declare const RIG = "\uE740";
export declare const RIGHT_JOIN = "\uE7A5";
export declare const RING = "\uE6F2";
export declare const ROTATE_DOCUMENT = "\uE6E1";
export declare const ROTATE_PAGE = "\uE6E2";
export declare const SATELLITE = "\uE76B";
export declare const SAVED = "\uE6B6";
export declare const SCATTER_PLOT = "\uE73E";
export declare const SEARCH = "\uE64B";
export declare const SEARCH_AROUND = "\uE608";
export declare const SEARCH_TEMPLATE = "\uE628";
export declare const SEARCH_TEXT = "\uE663";
export declare const SEGMENTED_CONTROL = "\uE6EC";
export declare const SELECT = "\uE616";
export declare const SELECTION = "\u29BF";
export declare const SEND_TO = "\uE66E";
export declare const SEND_TO_GRAPH = "\uE736";
export declare const SEND_TO_MAP = "\uE737";
export declare const SERIES_ADD = "\uE796";
export declare const SERIES_CONFIGURATION = "\uE79A";
export declare const SERIES_DERIVED = "\uE799";
export declare const SERIES_FILTERED = "\uE798";
export declare const SERIES_SEARCH = "\uE797";
export declare const SETTINGS = "\uE6A2";
export declare const SHARE = "\uE62E";
export declare const SHIELD = "\uE7B2";
export declare const SHOP = "\uE6C2";
export declare const SHOPPING_CART = "\uE6C1";
export declare const SIGNAL_SEARCH = "\uE909";
export declare const SIM_CARD = "\uE718";
export declare const SLASH = "\uE769";
export declare const SMALL_CROSS = "\uE6D7";
export declare const SMALL_MINUS = "\uE70E";
export declare const SMALL_PLUS = "\uE70D";
export declare const SMALL_TICK = "\uE6D8";
export declare const SNOWFLAKE = "\uE7B6";
export declare const SOCIAL_MEDIA = "\uE671";
export declare const SORT = "\uE64F";
export declare const SORT_ALPHABETICAL = "\uE64D";
export declare const SORT_ALPHABETICAL_DESC = "\uE6C8";
export declare const SORT_ASC = "\uE6D5";
export declare const SORT_DESC = "\uE6D6";
export declare const SORT_NUMERICAL = "\uE64E";
export declare const SORT_NUMERICAL_DESC = "\uE6C9";
export declare const SPLIT_COLUMNS = "\uE750";
export declare const SQUARE = "\uE686";
export declare const STACKED_CHART = "\uE6E7";
export declare const STAR = "\u2605";
export declare const STAR_EMPTY = "\u2606";
export declare const STEP_BACKWARD = "\uE6A7";
export declare const STEP_CHART = "\uE70F";
export declare const STEP_FORWARD = "\uE6AD";
export declare const STOP = "\uE6AA";
export declare const STOPWATCH = "\uE901";
export declare const STRIKETHROUGH = "\uE7A6";
export declare const STYLE = "\uE601";
export declare const SWAP_HORIZONTAL = "\uE745";
export declare const SWAP_VERTICAL = "\uE744";
export declare const SYMBOL_CIRCLE = "\uE72E";
export declare const SYMBOL_CROSS = "\uE731";
export declare const SYMBOL_DIAMOND = "\uE730";
export declare const SYMBOL_SQUARE = "\uE72F";
export declare const SYMBOL_TRIANGLE_DOWN = "\uE733";
export declare const SYMBOL_TRIANGLE_UP = "\uE732";
export declare const TAG = "\uE61C";
export declare const TAKE_ACTION = "\uE6CA";
export declare const TAXI = "\uE79E";
export declare const TEXT_HIGHLIGHT = "\uE6DD";
export declare const TH = "\uE667";
export declare const TH_DERIVED = "\uE669";
export declare const TH_DISCONNECT = "\uE7D8";
export declare const TH_FILTERED = "\uE7C6";
export declare const TH_LIST = "\uE668";
export declare const THUMBS_DOWN = "\uE6BE";
export declare const THUMBS_UP = "\uE6BD";
export declare const TICK = "\u2713";
export declare const TICK_CIRCLE = "\uE779";
export declare const TIME = "\u23F2";
export declare const TIMELINE_AREA_CHART = "\uE6CD";
export declare const TIMELINE_BAR_CHART = "\uE620";
export declare const TIMELINE_EVENTS = "\uE61E";
export declare const TIMELINE_LINE_CHART = "\uE61F";
export declare const TINT = "\uE6B2";
export declare const TORCH = "\uE677";
export declare const TRACTOR = "\uE90C";
export declare const TRAIN = "\uE79F";
export declare const TRANSLATE = "\uE759";
export declare const TRASH = "\uE63B";
export declare const TREE = "\uE7B7";
export declare const TRENDING_DOWN = "\uE71A";
export declare const TRENDING_UP = "\uE719";
export declare const TRUCK = "\uE90B";
export declare const TWO_COLUMNS = "\uE657";
export declare const UNARCHIVE = "\uE906";
export declare const UNDERLINE = "\u2381";
export declare const UNDO = "\u238C";
export declare const UNGROUP_OBJECTS = "\uE688";
export declare const UNKNOWN_VEHICLE = "\uE73D";
export declare const UNLOCK = "\uE626";
export declare const UNPIN = "\uE650";
export declare const UNRESOLVE = "\uE679";
export declare const UPDATED = "\uE7A7";
export declare const UPLOAD = "\uE68F";
export declare const USER = "\uE627";
export declare const VARIABLE = "\uE6F5";
export declare const VERTICAL_BAR_CHART_ASC = "\uE75B";
export declare const VERTICAL_BAR_CHART_DESC = "\uE71C";
export declare const VERTICAL_DISTRIBUTION = "\uE721";
export declare const VIDEO = "\uE6A0";
export declare const VOLUME_DOWN = "\uE6A4";
export declare const VOLUME_OFF = "\uE6A3";
export declare const VOLUME_UP = "\uE6A5";
export declare const WALK = "\uE79D";
export declare const WARNING_SIGN = "\uE647";
export declare const WATERFALL_CHART = "\uE6E6";
export declare const WIDGET = "\uE678";
export declare const WIDGET_BUTTON = "\uE790";
export declare const WIDGET_FOOTER = "\uE792";
export declare const WIDGET_HEADER = "\uE791";
export declare const WRENCH = "\uE734";
export declare const ZOOM_IN = "\uE641";
export declare const ZOOM_OUT = "\uE642";
export declare const ZOOM_TO_FIT = "\uE67B";
