export declare const CLAMP_MIN_MAX: string;
export declare const ALERT_WARN_CANCEL_PROPS: string;
export declare const ALERT_WARN_CANCEL_ESCAPE_KEY: string;
export declare const ALERT_WARN_CANCEL_OUTSIDE_CLICK: string;
export declare const COLLAPSIBLE_LIST_INVALID_CHILD: string;
export declare const CONTEXTMENU_WARN_DECORATOR_NO_METHOD: string;
export declare const CONTEXTMENU_WARN_DECORATOR_NEEDS_REACT_ELEMENT: string;
export declare const HOTKEYS_HOTKEY_CHILDREN: string;
export declare const HOTKEYS_WARN_DECORATOR_NO_METHOD: string;
export declare const HOTKEYS_WARN_DECORATOR_NEEDS_REACT_ELEMENT: string;
export declare const HOTKEYS_TARGET2_CHILDREN_LOCAL_HOTKEYS: string;
export declare const INPUT_WARN_LEFT_ELEMENT_LEFT_ICON_MUTEX: string;
export declare const NUMERIC_INPUT_MIN_MAX: string;
export declare const NUMERIC_INPUT_MINOR_STEP_SIZE_BOUND: string;
export declare const NUMERIC_INPUT_MAJOR_STEP_SIZE_BOUND: string;
export declare const NUMERIC_INPUT_MINOR_STEP_SIZE_NON_POSITIVE: string;
export declare const NUMERIC_INPUT_MAJOR_STEP_SIZE_NON_POSITIVE: string;
export declare const NUMERIC_INPUT_STEP_SIZE_NON_POSITIVE: string;
export declare const NUMERIC_INPUT_CONTROLLED_VALUE_INVALID: string;
export declare const PANEL_STACK_INITIAL_PANEL_STACK_MUTEX: string;
export declare const PANEL_STACK_REQUIRES_PANEL: string;
export declare const OVERFLOW_LIST_OBSERVE_PARENTS_CHANGED: string;
export declare const POPOVER_REQUIRES_TARGET: string;
export declare const POPOVER_HAS_BACKDROP_INTERACTION: string;
export declare const POPOVER_WARN_TOO_MANY_CHILDREN: string;
export declare const POPOVER_WARN_DOUBLE_CONTENT: string;
export declare const POPOVER_WARN_DOUBLE_TARGET: string;
export declare const POPOVER_WARN_EMPTY_CONTENT: string;
export declare const POPOVER_WARN_HAS_BACKDROP_INLINE: string;
export declare const POPOVER_WARN_PLACEMENT_AND_POSITION_MUTEX: string;
export declare const POPOVER_WARN_UNCONTROLLED_ONINTERACTION: string;
export declare const PORTAL_CONTEXT_CLASS_NAME_STRING: string;
export declare const RADIOGROUP_WARN_CHILDREN_OPTIONS_MUTEX: string;
export declare const SLIDER_ZERO_STEP: string;
export declare const SLIDER_ZERO_LABEL_STEP: string;
export declare const RANGESLIDER_NULL_VALUE: string;
export declare const MULTISLIDER_INVALID_CHILD: string;
export declare const MULTISLIDER_WARN_LABEL_STEP_SIZE_LABEL_VALUES_MUTEX: string;
export declare const SPINNER_WARN_CLASSES_SIZE: string;
export declare const TOASTER_CREATE_NULL: string;
export declare const TOASTER_WARN_INLINE: string;
export declare const DIALOG_WARN_NO_HEADER_ICON: string;
export declare const DIALOG_WARN_NO_HEADER_CLOSE_BUTTON: string;
export declare const DRAWER_VERTICAL_IS_IGNORED: string;
export declare const DRAWER_ANGLE_POSITIONS_ARE_CASTED: string;
export declare const TOASTER_MAX_TOASTS_INVALID: string;
