export = ProcessNodeDefinitions;
declare function ProcessNodeDefinitions(): {
    processDefaultNode: (node: any, children: any, index: number) => any;
};
//# sourceMappingURL=process-node-definitions.d.ts.map
