export = Html2ReactParser;
declare function Html2ReactParser(options?: any): {
    parse: (html: any) => any;
    parseWithInstructions: (html: any, isValidNode: any, processingInstructions: any, preprocessingInstructions?: any) => any;
};
//# sourceMappingURL=parser.d.ts.map
