export declare const ADD = "add";
export declare const ADD_CLIP = "add-clip";
export declare const ADD_COLUMN_LEFT = "add-column-left";
export declare const ADD_COLUMN_RIGHT = "add-column-right";
export declare const ADD_LOCATION = "add-location";
export declare const ADD_ROW_BOTTOM = "add-row-bottom";
export declare const ADD_ROW_TOP = "add-row-top";
export declare const ADD_TO_ARTIFACT = "add-to-artifact";
export declare const ADD_TO_FOLDER = "add-to-folder";
export declare const AIRPLANE = "airplane";
export declare const ALIGN_CENTER = "align-center";
export declare const ALIGN_JUSTIFY = "align-justify";
export declare const ALIGN_LEFT = "align-left";
export declare const ALIGN_RIGHT = "align-right";
export declare const ALIGNMENT_BOTTOM = "alignment-bottom";
export declare const ALIGNMENT_HORIZONTAL_CENTER = "alignment-horizontal-center";
export declare const ALIGNMENT_LEFT = "alignment-left";
export declare const ALIGNMENT_RIGHT = "alignment-right";
export declare const ALIGNMENT_TOP = "alignment-top";
export declare const ALIGNMENT_VERTICAL_CENTER = "alignment-vertical-center";
export declare const ANNOTATION = "annotation";
export declare const ANTENNA = "antenna";
export declare const APP_HEADER = "app-header";
export declare const APPLICATION = "application";
export declare const APPLICATIONS = "applications";
export declare const ARCHIVE = "archive";
export declare const AREA_OF_INTEREST = "area-of-interest";
export declare const ARRAY = "array";
export declare const ARRAY_BOOLEAN = "array-boolean";
export declare const ARRAY_DATE = "array-date";
export declare const ARRAY_NUMERIC = "array-numeric";
export declare const ARRAY_STRING = "array-string";
export declare const ARRAY_TIMESTAMP = "array-timestamp";
export declare const ARROW_BOTTOM_LEFT = "arrow-bottom-left";
export declare const ARROW_BOTTOM_RIGHT = "arrow-bottom-right";
export declare const ARROW_DOWN = "arrow-down";
export declare const ARROW_LEFT = "arrow-left";
export declare const ARROW_RIGHT = "arrow-right";
export declare const ARROW_TOP_LEFT = "arrow-top-left";
export declare const ARROW_TOP_RIGHT = "arrow-top-right";
export declare const ARROW_UP = "arrow-up";
export declare const ARROWS_HORIZONTAL = "arrows-horizontal";
export declare const ARROWS_VERTICAL = "arrows-vertical";
export declare const ASTERISK = "asterisk";
export declare const AUTOMATIC_UPDATES = "automatic-updates";
export declare const BACKLINK = "backlink";
export declare const BADGE = "badge";
export declare const BAN_CIRCLE = "ban-circle";
export declare const BANK_ACCOUNT = "bank-account";
export declare const BARCODE = "barcode";
export declare const BLANK = "blank";
export declare const BLOCKED_PERSON = "blocked-person";
export declare const BOLD = "bold";
export declare const BOOK = "book";
export declare const BOOKMARK = "bookmark";
export declare const BOX = "box";
export declare const BRIEFCASE = "briefcase";
export declare const BRING_DATA = "bring-data";
export declare const BUGGY = "buggy";
export declare const BUILD = "build";
export declare const CALCULATOR = "calculator";
export declare const CALENDAR = "calendar";
export declare const CAMERA = "camera";
export declare const CARET_DOWN = "caret-down";
export declare const CARET_LEFT = "caret-left";
export declare const CARET_RIGHT = "caret-right";
export declare const CARET_UP = "caret-up";
export declare const CARGO_SHIP = "cargo-ship";
export declare const CELL_TOWER = "cell-tower";
export declare const CHANGES = "changes";
export declare const CHART = "chart";
export declare const CHAT = "chat";
export declare const CHEVRON_BACKWARD = "chevron-backward";
export declare const CHEVRON_DOWN = "chevron-down";
export declare const CHEVRON_FORWARD = "chevron-forward";
export declare const CHEVRON_LEFT = "chevron-left";
export declare const CHEVRON_RIGHT = "chevron-right";
export declare const CHEVRON_UP = "chevron-up";
export declare const CIRCLE = "circle";
export declare const CIRCLE_ARROW_DOWN = "circle-arrow-down";
export declare const CIRCLE_ARROW_LEFT = "circle-arrow-left";
export declare const CIRCLE_ARROW_RIGHT = "circle-arrow-right";
export declare const CIRCLE_ARROW_UP = "circle-arrow-up";
export declare const CITATION = "citation";
export declare const CLEAN = "clean";
export declare const CLIP = "clip";
export declare const CLIPBOARD = "clipboard";
export declare const CLOUD = "cloud";
export declare const CLOUD_DOWNLOAD = "cloud-download";
export declare const CLOUD_UPLOAD = "cloud-upload";
export declare const CODE = "code";
export declare const CODE_BLOCK = "code-block";
export declare const COG = "cog";
export declare const COLLAPSE_ALL = "collapse-all";
export declare const COLUMN_LAYOUT = "column-layout";
export declare const COMMENT = "comment";
export declare const COMPARISON = "comparison";
export declare const COMPASS = "compass";
export declare const COMPRESSED = "compressed";
export declare const CONFIRM = "confirm";
export declare const CONSOLE = "console";
export declare const CONTRAST = "contrast";
export declare const CONTROL = "control";
export declare const CREDIT_CARD = "credit-card";
export declare const CROSS = "cross";
export declare const CROWN = "crown";
export declare const CUBE = "cube";
export declare const CUBE_ADD = "cube-add";
export declare const CUBE_REMOVE = "cube-remove";
export declare const CURVED_RANGE_CHART = "curved-range-chart";
export declare const CUT = "cut";
export declare const CYCLE = "cycle";
export declare const DASHBOARD = "dashboard";
export declare const DATA_CONNECTION = "data-connection";
export declare const DATA_LINEAGE = "data-lineage";
export declare const DATABASE = "database";
export declare const DELETE = "delete";
export declare const DELTA = "delta";
export declare const DERIVE_COLUMN = "derive-column";
export declare const DESKTOP = "desktop";
export declare const DIAGNOSIS = "diagnosis";
export declare const DIAGRAM_TREE = "diagram-tree";
export declare const DIRECTION_LEFT = "direction-left";
export declare const DIRECTION_RIGHT = "direction-right";
export declare const DISABLE = "disable";
export declare const DOCUMENT = "document";
export declare const DOCUMENT_OPEN = "document-open";
export declare const DOCUMENT_SHARE = "document-share";
export declare const DOLLAR = "dollar";
export declare const DOT = "dot";
export declare const DOUBLE_CARET_HORIZONTAL = "double-caret-horizontal";
export declare const DOUBLE_CARET_VERTICAL = "double-caret-vertical";
export declare const DOUBLE_CHEVRON_DOWN = "double-chevron-down";
export declare const DOUBLE_CHEVRON_LEFT = "double-chevron-left";
export declare const DOUBLE_CHEVRON_RIGHT = "double-chevron-right";
export declare const DOUBLE_CHEVRON_UP = "double-chevron-up";
export declare const DOUGHNUT_CHART = "doughnut-chart";
export declare const DOWNLOAD = "download";
export declare const DRAG_HANDLE_HORIZONTAL = "drag-handle-horizontal";
export declare const DRAG_HANDLE_VERTICAL = "drag-handle-vertical";
export declare const DRAW = "draw";
export declare const DRAWER_LEFT = "drawer-left";
export declare const DRAWER_LEFT_FILLED = "drawer-left-filled";
export declare const DRAWER_RIGHT = "drawer-right";
export declare const DRAWER_RIGHT_FILLED = "drawer-right-filled";
export declare const DRIVE_TIME = "drive-time";
export declare const DUPLICATE = "duplicate";
export declare const EDIT = "edit";
export declare const EJECT = "eject";
export declare const EMOJI = "emoji";
export declare const ENDORSED = "endorsed";
export declare const ENVELOPE = "envelope";
export declare const EQUALS = "equals";
export declare const ERASER = "eraser";
export declare const ERROR = "error";
export declare const EURO = "euro";
export declare const EXCHANGE = "exchange";
export declare const EXCLUDE_ROW = "exclude-row";
export declare const EXPAND_ALL = "expand-all";
export declare const EXPORT = "export";
export declare const EYE_OFF = "eye-off";
export declare const EYE_ON = "eye-on";
export declare const EYE_OPEN = "eye-open";
export declare const FAST_BACKWARD = "fast-backward";
export declare const FAST_FORWARD = "fast-forward";
export declare const FEED = "feed";
export declare const FEED_SUBSCRIBED = "feed-subscribed";
export declare const FILM = "film";
export declare const FILTER = "filter";
export declare const FILTER_KEEP = "filter-keep";
export declare const FILTER_LIST = "filter-list";
export declare const FILTER_OPEN = "filter-open";
export declare const FILTER_REMOVE = "filter-remove";
export declare const FLAG = "flag";
export declare const FLAME = "flame";
export declare const FLASH = "flash";
export declare const FLOPPY_DISK = "floppy-disk";
export declare const FLOW_BRANCH = "flow-branch";
export declare const FLOW_END = "flow-end";
export declare const FLOW_LINEAR = "flow-linear";
export declare const FLOW_REVIEW = "flow-review";
export declare const FLOW_REVIEW_BRANCH = "flow-review-branch";
export declare const FLOWS = "flows";
export declare const FOLDER_CLOSE = "folder-close";
export declare const FOLDER_NEW = "folder-new";
export declare const FOLDER_OPEN = "folder-open";
export declare const FOLDER_SHARED = "folder-shared";
export declare const FOLDER_SHARED_OPEN = "folder-shared-open";
export declare const FOLLOWER = "follower";
export declare const FOLLOWING = "following";
export declare const FONT = "font";
export declare const FORK = "fork";
export declare const FORM = "form";
export declare const FULL_CIRCLE = "full-circle";
export declare const FULL_STACKED_CHART = "full-stacked-chart";
export declare const FULLSCREEN = "fullscreen";
export declare const FUNCTION = "function";
export declare const GANTT_CHART = "gantt-chart";
export declare const GEOFENCE = "geofence";
export declare const GEOLOCATION = "geolocation";
export declare const GEOSEARCH = "geosearch";
export declare const GIT_BRANCH = "git-branch";
export declare const GIT_COMMIT = "git-commit";
export declare const GIT_MERGE = "git-merge";
export declare const GIT_NEW_BRANCH = "git-new-branch";
export declare const GIT_PULL = "git-pull";
export declare const GIT_PUSH = "git-push";
export declare const GIT_REPO = "git-repo";
export declare const GLASS = "glass";
export declare const GLOBE = "globe";
export declare const GLOBE_NETWORK = "globe-network";
export declare const GRAPH = "graph";
export declare const GRAPH_REMOVE = "graph-remove";
export declare const GREATER_THAN = "greater-than";
export declare const GREATER_THAN_OR_EQUAL_TO = "greater-than-or-equal-to";
export declare const GRID = "grid";
export declare const GRID_VIEW = "grid-view";
export declare const GROUP_OBJECTS = "group-objects";
export declare const GROUPED_BAR_CHART = "grouped-bar-chart";
export declare const HAND = "hand";
export declare const HAND_DOWN = "hand-down";
export declare const HAND_LEFT = "hand-left";
export declare const HAND_RIGHT = "hand-right";
export declare const HAND_UP = "hand-up";
export declare const HAT = "hat";
export declare const HEADER = "header";
export declare const HEADER_ONE = "header-one";
export declare const HEADER_THREE = "header-three";
export declare const HEADER_TWO = "header-two";
export declare const HEADSET = "headset";
export declare const HEART = "heart";
export declare const HEART_BROKEN = "heart-broken";
export declare const HEAT_GRID = "heat-grid";
export declare const HEATMAP = "heatmap";
export declare const HELICOPTER = "helicopter";
export declare const HELP = "help";
export declare const HELPER_MANAGEMENT = "helper-management";
export declare const HIGH_PRIORITY = "high-priority";
export declare const HIGHLIGHT = "highlight";
export declare const HISTORY = "history";
export declare const HOME = "home";
export declare const HORIZONTAL_BAR_CHART = "horizontal-bar-chart";
export declare const HORIZONTAL_BAR_CHART_ASC = "horizontal-bar-chart-asc";
export declare const HORIZONTAL_BAR_CHART_DESC = "horizontal-bar-chart-desc";
export declare const HORIZONTAL_DISTRIBUTION = "horizontal-distribution";
export declare const HURRICANE = "hurricane";
export declare const ID_NUMBER = "id-number";
export declare const IMAGE_ROTATE_LEFT = "image-rotate-left";
export declare const IMAGE_ROTATE_RIGHT = "image-rotate-right";
export declare const IMPORT = "import";
export declare const INBOX = "inbox";
export declare const INBOX_FILTERED = "inbox-filtered";
export declare const INBOX_GEO = "inbox-geo";
export declare const INBOX_SEARCH = "inbox-search";
export declare const INBOX_UPDATE = "inbox-update";
export declare const INFO_SIGN = "info-sign";
export declare const INHERITANCE = "inheritance";
export declare const INHERITED_GROUP = "inherited-group";
export declare const INNER_JOIN = "inner-join";
export declare const INSERT = "insert";
export declare const INTERSECTION = "intersection";
export declare const IP_ADDRESS = "ip-address";
export declare const ISSUE = "issue";
export declare const ISSUE_CLOSED = "issue-closed";
export declare const ISSUE_NEW = "issue-new";
export declare const ITALIC = "italic";
export declare const JOIN_TABLE = "join-table";
export declare const KEY = "key";
export declare const KEY_BACKSPACE = "key-backspace";
export declare const KEY_COMMAND = "key-command";
export declare const KEY_CONTROL = "key-control";
export declare const KEY_DELETE = "key-delete";
export declare const KEY_ENTER = "key-enter";
export declare const KEY_ESCAPE = "key-escape";
export declare const KEY_OPTION = "key-option";
export declare const KEY_SHIFT = "key-shift";
export declare const KEY_TAB = "key-tab";
export declare const KNOWN_VEHICLE = "known-vehicle";
export declare const LAB_TEST = "lab-test";
export declare const LABEL = "label";
export declare const LAYER = "layer";
export declare const LAYER_OUTLINE = "layer-outline";
export declare const LAYERS = "layers";
export declare const LAYOUT = "layout";
export declare const LAYOUT_AUTO = "layout-auto";
export declare const LAYOUT_BALLOON = "layout-balloon";
export declare const LAYOUT_CIRCLE = "layout-circle";
export declare const LAYOUT_GRID = "layout-grid";
export declare const LAYOUT_GROUP_BY = "layout-group-by";
export declare const LAYOUT_HIERARCHY = "layout-hierarchy";
export declare const LAYOUT_LINEAR = "layout-linear";
export declare const LAYOUT_SKEW_GRID = "layout-skew-grid";
export declare const LAYOUT_SORTED_CLUSTERS = "layout-sorted-clusters";
export declare const LEARNING = "learning";
export declare const LEFT_JOIN = "left-join";
export declare const LESS_THAN = "less-than";
export declare const LESS_THAN_OR_EQUAL_TO = "less-than-or-equal-to";
export declare const LIFESAVER = "lifesaver";
export declare const LIGHTBULB = "lightbulb";
export declare const LIGHTNING = "lightning";
export declare const LINK = "link";
export declare const LIST = "list";
export declare const LIST_COLUMNS = "list-columns";
export declare const LIST_DETAIL_VIEW = "list-detail-view";
export declare const LOCATE = "locate";
export declare const LOCK = "lock";
export declare const LOG_IN = "log-in";
export declare const LOG_OUT = "log-out";
export declare const MANUAL = "manual";
export declare const MANUALLY_ENTERED_DATA = "manually-entered-data";
export declare const MANY_TO_MANY = "many-to-many";
export declare const MANY_TO_ONE = "many-to-one";
export declare const MAP = "map";
export declare const MAP_CREATE = "map-create";
export declare const MAP_MARKER = "map-marker";
export declare const MAXIMIZE = "maximize";
export declare const MEDIA = "media";
export declare const MENU = "menu";
export declare const MENU_CLOSED = "menu-closed";
export declare const MENU_OPEN = "menu-open";
export declare const MERGE_COLUMNS = "merge-columns";
export declare const MERGE_LINKS = "merge-links";
export declare const MINIMIZE = "minimize";
export declare const MINUS = "minus";
export declare const MOBILE_PHONE = "mobile-phone";
export declare const MOBILE_VIDEO = "mobile-video";
export declare const MODAL = "modal";
export declare const MODAL_FILLED = "modal-filled";
export declare const MOON = "moon";
export declare const MORE = "more";
export declare const MOUNTAIN = "mountain";
export declare const MOVE = "move";
export declare const MUGSHOT = "mugshot";
export declare const MULTI_SELECT = "multi-select";
export declare const MUSIC = "music";
export declare const NEST = "nest";
export declare const NEW_DRAWING = "new-drawing";
export declare const NEW_GRID_ITEM = "new-grid-item";
export declare const NEW_LAYER = "new-layer";
export declare const NEW_LAYERS = "new-layers";
export declare const NEW_LINK = "new-link";
export declare const NEW_OBJECT = "new-object";
export declare const NEW_PERSON = "new-person";
export declare const NEW_PRESCRIPTION = "new-prescription";
export declare const NEW_TEXT_BOX = "new-text-box";
export declare const NINJA = "ninja";
export declare const NOT_EQUAL_TO = "not-equal-to";
export declare const NOTIFICATIONS = "notifications";
export declare const NOTIFICATIONS_SNOOZE = "notifications-snooze";
export declare const NOTIFICATIONS_UPDATED = "notifications-updated";
export declare const NUMBERED_LIST = "numbered-list";
export declare const NUMERICAL = "numerical";
export declare const OFFICE = "office";
export declare const OFFLINE = "offline";
export declare const OIL_FIELD = "oil-field";
export declare const ONE_COLUMN = "one-column";
export declare const ONE_TO_MANY = "one-to-many";
export declare const ONE_TO_ONE = "one-to-one";
export declare const OUTDATED = "outdated";
export declare const PAGE_LAYOUT = "page-layout";
export declare const PANEL_STATS = "panel-stats";
export declare const PANEL_TABLE = "panel-table";
export declare const PAPERCLIP = "paperclip";
export declare const PARAGRAPH = "paragraph";
export declare const PATH = "path";
export declare const PATH_SEARCH = "path-search";
export declare const PAUSE = "pause";
export declare const PEOPLE = "people";
export declare const PERCENTAGE = "percentage";
export declare const PERSON = "person";
export declare const PHONE = "phone";
export declare const PIE_CHART = "pie-chart";
export declare const PIN = "pin";
export declare const PIVOT = "pivot";
export declare const PIVOT_TABLE = "pivot-table";
export declare const PLAY = "play";
export declare const PLUS = "plus";
export declare const POLYGON_FILTER = "polygon-filter";
export declare const POWER = "power";
export declare const PREDICTIVE_ANALYSIS = "predictive-analysis";
export declare const PRESCRIPTION = "prescription";
export declare const PRESENTATION = "presentation";
export declare const PRINT = "print";
export declare const PROJECTS = "projects";
export declare const PROPERTIES = "properties";
export declare const PROPERTY = "property";
export declare const PUBLISH_FUNCTION = "publish-function";
export declare const PULSE = "pulse";
export declare const RAIN = "rain";
export declare const RANDOM = "random";
export declare const RECORD = "record";
export declare const RECTANGLE = "rectangle";
export declare const REDO = "redo";
export declare const REFRESH = "refresh";
export declare const REGRESSION_CHART = "regression-chart";
export declare const REMOVE = "remove";
export declare const REMOVE_COLUMN = "remove-column";
export declare const REMOVE_COLUMN_LEFT = "remove-column-left";
export declare const REMOVE_COLUMN_RIGHT = "remove-column-right";
export declare const REMOVE_ROW_BOTTOM = "remove-row-bottom";
export declare const REMOVE_ROW_TOP = "remove-row-top";
export declare const REPEAT = "repeat";
export declare const RESET = "reset";
export declare const RESOLVE = "resolve";
export declare const RIG = "rig";
export declare const RIGHT_JOIN = "right-join";
export declare const RING = "ring";
export declare const ROCKET = "rocket";
export declare const ROCKET_SLANT = "rocket-slant";
export declare const ROTATE_DOCUMENT = "rotate-document";
export declare const ROTATE_PAGE = "rotate-page";
export declare const ROUTE = "route";
export declare const SATELLITE = "satellite";
export declare const SAVED = "saved";
export declare const SCATTER_PLOT = "scatter-plot";
export declare const SEARCH = "search";
export declare const SEARCH_AROUND = "search-around";
export declare const SEARCH_TEMPLATE = "search-template";
export declare const SEARCH_TEXT = "search-text";
export declare const SEGMENTED_CONTROL = "segmented-control";
export declare const SELECT = "select";
export declare const SELECTION = "selection";
export declare const SEND_MESSAGE = "send-message";
export declare const SEND_TO = "send-to";
export declare const SEND_TO_GRAPH = "send-to-graph";
export declare const SEND_TO_MAP = "send-to-map";
export declare const SERIES_ADD = "series-add";
export declare const SERIES_CONFIGURATION = "series-configuration";
export declare const SERIES_DERIVED = "series-derived";
export declare const SERIES_FILTERED = "series-filtered";
export declare const SERIES_SEARCH = "series-search";
export declare const SETTINGS = "settings";
export declare const SHAPES = "shapes";
export declare const SHARE = "share";
export declare const SHARED_FILTER = "shared-filter";
export declare const SHIELD = "shield";
export declare const SHIP = "ship";
export declare const SHOP = "shop";
export declare const SHOPPING_CART = "shopping-cart";
export declare const SIGNAL_SEARCH = "signal-search";
export declare const SIM_CARD = "sim-card";
export declare const SLASH = "slash";
export declare const SMALL_CROSS = "small-cross";
export declare const SMALL_MINUS = "small-minus";
export declare const SMALL_PLUS = "small-plus";
export declare const SMALL_SQUARE = "small-square";
export declare const SMALL_TICK = "small-tick";
export declare const SNOWFLAKE = "snowflake";
export declare const SOCIAL_MEDIA = "social-media";
export declare const SORT = "sort";
export declare const SORT_ALPHABETICAL = "sort-alphabetical";
export declare const SORT_ALPHABETICAL_DESC = "sort-alphabetical-desc";
export declare const SORT_ASC = "sort-asc";
export declare const SORT_DESC = "sort-desc";
export declare const SORT_NUMERICAL = "sort-numerical";
export declare const SORT_NUMERICAL_DESC = "sort-numerical-desc";
export declare const SPLIT_COLUMNS = "split-columns";
export declare const SQUARE = "square";
export declare const STACKED_CHART = "stacked-chart";
export declare const STADIUM_GEOMETRY = "stadium-geometry";
export declare const STAR = "star";
export declare const STAR_EMPTY = "star-empty";
export declare const STEP_BACKWARD = "step-backward";
export declare const STEP_CHART = "step-chart";
export declare const STEP_FORWARD = "step-forward";
export declare const STOP = "stop";
export declare const STOPWATCH = "stopwatch";
export declare const STRIKETHROUGH = "strikethrough";
export declare const STYLE = "style";
export declare const SWAP_HORIZONTAL = "swap-horizontal";
export declare const SWAP_VERTICAL = "swap-vertical";
export declare const SWITCH = "switch";
export declare const SYMBOL_CIRCLE = "symbol-circle";
export declare const SYMBOL_CROSS = "symbol-cross";
export declare const SYMBOL_DIAMOND = "symbol-diamond";
export declare const SYMBOL_RECTANGLE = "symbol-rectangle";
export declare const SYMBOL_SQUARE = "symbol-square";
export declare const SYMBOL_TRIANGLE_DOWN = "symbol-triangle-down";
export declare const SYMBOL_TRIANGLE_UP = "symbol-triangle-up";
export declare const SYRINGE = "syringe";
export declare const TAG = "tag";
export declare const TAKE_ACTION = "take-action";
export declare const TANK = "tank";
export declare const TARGET = "target";
export declare const TAXI = "taxi";
export declare const TEMPERATURE = "temperature";
export declare const TEXT_HIGHLIGHT = "text-highlight";
export declare const TH = "th";
export declare const TH_DERIVED = "th-derived";
export declare const TH_DISCONNECT = "th-disconnect";
export declare const TH_FILTERED = "th-filtered";
export declare const TH_LIST = "th-list";
export declare const THIRD_PARTY = "third-party";
export declare const THUMBS_DOWN = "thumbs-down";
export declare const THUMBS_UP = "thumbs-up";
export declare const TICK = "tick";
export declare const TICK_CIRCLE = "tick-circle";
export declare const TIME = "time";
export declare const TIMELINE_AREA_CHART = "timeline-area-chart";
export declare const TIMELINE_BAR_CHART = "timeline-bar-chart";
export declare const TIMELINE_EVENTS = "timeline-events";
export declare const TIMELINE_LINE_CHART = "timeline-line-chart";
export declare const TINT = "tint";
export declare const TORCH = "torch";
export declare const TRACTOR = "tractor";
export declare const TRAIN = "train";
export declare const TRANSLATE = "translate";
export declare const TRASH = "trash";
export declare const TREE = "tree";
export declare const TRENDING_DOWN = "trending-down";
export declare const TRENDING_UP = "trending-up";
export declare const TRUCK = "truck";
export declare const TWO_COLUMNS = "two-columns";
export declare const UNARCHIVE = "unarchive";
export declare const UNDERLINE = "underline";
export declare const UNDO = "undo";
export declare const UNGROUP_OBJECTS = "ungroup-objects";
export declare const UNKNOWN_VEHICLE = "unknown-vehicle";
export declare const UNLOCK = "unlock";
export declare const UNPIN = "unpin";
export declare const UNRESOLVE = "unresolve";
export declare const UPDATED = "updated";
export declare const UPLOAD = "upload";
export declare const USER = "user";
export declare const VARIABLE = "variable";
export declare const VERTICAL_BAR_CHART_ASC = "vertical-bar-chart-asc";
export declare const VERTICAL_BAR_CHART_DESC = "vertical-bar-chart-desc";
export declare const VERTICAL_DISTRIBUTION = "vertical-distribution";
export declare const VIDEO = "video";
export declare const VIRUS = "virus";
export declare const VOLUME_DOWN = "volume-down";
export declare const VOLUME_OFF = "volume-off";
export declare const VOLUME_UP = "volume-up";
export declare const WALK = "walk";
export declare const WARNING_SIGN = "warning-sign";
export declare const WATERFALL_CHART = "waterfall-chart";
export declare const WAVES = "waves";
export declare const WIDGET = "widget";
export declare const WIDGET_BUTTON = "widget-button";
export declare const WIDGET_FOOTER = "widget-footer";
export declare const WIDGET_HEADER = "widget-header";
export declare const WIND = "wind";
export declare const WRENCH = "wrench";
export declare const ZOOM_IN = "zoom-in";
export declare const ZOOM_OUT = "zoom-out";
export declare const ZOOM_TO_FIT = "zoom-to-fit";
