/// <reference types="react" />
interface Props {
    data: string;
    mediaType: "text/plain";
}
export declare const Plain: {
    (props: Props): JSX.Element;
    defaultProps: {
        data: string;
        mediaType: string;
    };
    displayName: string;
};
export default Plain;
