"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _autoit = _interopRequireDefault(require("highlight.js/lib/languages/autoit"));

var _default = _autoit["default"];
exports["default"] = _default;