import { FC, HTMLAttributes } from "react";
export declare const Markdown: (props: any) => JSX.Element;
export declare const Delete: () => JSX.Element;
interface MutedProps extends HTMLAttributes<SVGElement> {
    muted?: boolean;
}
export declare const Commands: FC<MutedProps>;
export declare const AddCell: ({ below }: {
    below?: boolean | undefined;
}) => JSX.Element;
export declare const Clear: () => JSX.Element;
export declare const Play: () => JSX.Element;
export declare const More: () => JSX.Element;
export declare const File: (props: HTMLAttributes<SVGElement>) => JSX.Element;
export declare const Folder: (props: HTMLAttributes<SVGElement>) => JSX.Element;
export declare const Chevron: (props: HTMLAttributes<SVGElement>) => JSX.Element;
export declare const Eye: (props: HTMLAttributes<SVGElement>) => JSX.Element;
export {};
