import GlobalModel from '../../model/Global.js';
import ExtensionAPI from '../../core/ExtensionAPI.js';
import { Payload } from '../../util/types.js';
export declare function layoutCovers(ecModel: GlobalModel): void;
/**
 * Register the visual encoding if this modules required.
 */
export default function brushVisual(ecModel: GlobalModel, api: ExtensionAPI, payload: Payload): void;
