import * as React from "react";
export interface SourceProps {
    language?: string;
    children?: React.ReactNode;
    className?: string;
    theme?: "light" | "dark";
}
declare class BareSource extends React.Component<SourceProps> {
    static defaultProps: {
        children: string;
        language: string;
        className: string;
        theme: string;
    };
    render(): JSX.Element;
}
export declare const Source: import("styled-components").StyledComponent<typeof BareSource, any, SourceProps, never>;
export default Source;
