export declare const Collapse: ({ className }: {
    className?: string | undefined;
}) => JSX.Element;
export declare const Expand: () => JSX.Element;
export declare const ExpandSidebar: ({ isSidebarVisible, }: {
    isSidebarVisible?: boolean | undefined;
}) => JSX.Element;
export declare const Clear: () => JSX.Element;
export declare const Commands: () => JSX.Element;
export declare const Group: () => JSX.Element;
export declare const Pause: () => JSX.Element;
export declare const Play: () => JSX.Element;
export declare const Plus: () => JSX.Element;
