export declare type AlgoliaInsightsHit = {
    objectID: string;
    __autocomplete_indexName: string;
    __autocomplete_queryID: string;
    __autocomplete_algoliaCredentials: {
        appId: string;
        apiKey: string;
    };
};
