import React, { HTMLAttributes } from "react";
interface CellMenuProps extends React.FC<HTMLAttributes<HTMLDivElement>> {
    visible: boolean;
}
export declare type Props = React.FC<HTMLAttributes<HTMLUListElement>>;
interface MenuItemProps extends React.FC<HTMLAttributes<HTMLLIElement>> {
    className?: string;
    onClick?: () => void;
    focusWithin?: boolean;
    tabIndex?: number;
}
export declare const CellMenuSection: Props;
export declare class CellMenuItem extends React.PureComponent<MenuItemProps, {}> {
    element: any;
    handleKeypress: (e: KeyboardEvent) => void;
    handleFocus: () => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
}
export declare const CellMenu: React.FC<CellMenuProps>;
export {};
