import * as React from "react";
interface HighlighterProps {
    language: string;
    className: string;
    children?: React.ReactNode;
    theme?: "light" | "dark";
}
declare const Highlighter: {
    (props: HighlighterProps): JSX.Element;
    defaultProps: {
        theme: string;
        language: string;
        children: string;
        className: string;
    };
};
export default Highlighter;
