export = ProcessingInstructions;
declare function ProcessingInstructions(): {
    defaultProcessingInstructions: {
        shouldProcessNode: (node: any) => boolean;
        processNode: any;
    }[];
};
//# sourceMappingURL=processing-instructions.d.ts.map