import { Placement } from "popper.js";
import { PopoverPosition } from "./popoverSharedProps";
/**
 * Convert a position to a placement.
 *
 * @param position the position to convert
 */
export declare function positionToPlacement(position: PopoverPosition): Placement;
