import type { InternalHandlerBuilder } from './types';
export declare const buildBatchedActionsHandler: InternalHandlerBuilder<[
    actionShouldContinue: boolean,
    subscriptionExists: boolean
]>;
