declare const DarkTheme: import("styled-components").GlobalStyleComponent<{}, import("styled-components").DefaultTheme>;
declare const LightTheme: import("styled-components").GlobalStyleComponent<{}, import("styled-components").DefaultTheme>;
export { DarkTheme, LightTheme };
