import { HTMLAttributes, ReactChild, Component } from "react";
export interface Props extends HTMLAttributes<HTMLDivElement> {
    children?: ReactChild | JSX.Element[];
    onToggleFileBrowser: () => void;
    isSidebarVisible?: boolean;
}
export interface State {
    toolbarState: string;
}
export declare class Toolbar extends Component<Props, State> {
    constructor(props: Props);
    render(): JSX.Element;
}
