/*!
 * Copyright 2016 The ANTLR Project. All rights reserved.
 * Licensed under the BSD-3-Clause license. See LICENSE file in the project root for license information.
 */
export declare function isHighSurrogate(ch: number): boolean;
export declare function isLowSurrogate(ch: number): boolean;
export declare function isSupplementaryCodePoint(ch: number): boolean;
