/// <reference types="react" />
interface Props {
    data: string;
    mediaType: "image/png" | "image/jpeg" | "image/gif";
    metadata: {
        height?: number;
        width?: number;
    };
}
export declare function Image(props: Partial<Props>): JSX.Element;
export declare namespace Image {
    var defaultProps: {
        data: string;
        mediaType: string;
    };
}
export default Image;
