"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _phpdoc = _interopRequireDefault(require("refractor/lang/phpdoc.js"));

;
var _default = _phpdoc["default"];
exports["default"] = _default;